@@ -15,6 +15,7 @@ use crate::{
         position::MAX_POSITION,
         role::CrewRole,
         skill::{GameSkill, MAX_SKILL},
+        utils::is_default,
     },
 };
 use itertools::Itertools;
@@ -41,12 +42,15 @@ pub struct GameSummary {
     pub starting_at: Tick,
     pub ended_at: Option<Tick>,
     pub winner: Option<TeamId>,
+    #[serde(skip_serializing_if = "is_default")]
+    #[serde(default)]
+    pub is_network: bool,
 }
 
 impl GameSummary {
     pub fn from_game(game: &Game) -> GameSummary {
-        let mut home_quarters_score = [0 as u16; 4];
-        let mut away_quarters_score = [0 as u16; 4];
+        let mut home_quarters_score = [0_u16; 4];
+        let mut away_quarters_score = [0_u16; 4];
         for action in game.action_results.iter() {
             // We need to loop over every action to cover the case in which the game ends abrutly because one team is knocked out.
             // For quarters>1, we need to remove previous quarters score to get only the partial score of the quarter.
@@ -80,7 +84,7 @@ impl GameSummary {
         }
 
         Self {
-            id: game.id.clone(),
+            id: game.id,
             home_team_id: game.home_team_in_game.team_id,
             away_team_id: game.away_team_in_game.team_id,
             home_team_name: game.home_team_in_game.name.clone(),
@@ -94,6 +98,8 @@ impl GameSummary {
             starting_at: game.starting_at,
             ended_at: game.ended_at,
             winner: game.winner,
+            is_network: game.home_team_in_game.peer_id.is_some()
+                || game.away_team_in_game.peer_id.is_some(),
         }
     }
 }
@@ -143,8 +149,8 @@ impl<'game> Game {
 
         let bonus_attendance = home_team_in_game
             .players
-            .iter()
-            .map(|(_, player)| {
+            .values()
+            .map(|player| {
                 if player.special_trait == Some(Trait::Showpirate) {
                     player.reputation.value()
                 } else {
@@ -155,8 +161,8 @@ impl<'game> Game {
             / 100.0
             + away_team_in_game
                 .players
-                .iter()
-                .map(|(_, player)| {
+                .values()
+                .map(|player| {
                     if player.special_trait == Some(Trait::Showpirate) {
                         player.reputation.value()
                     } else {
@@ -185,11 +191,11 @@ impl<'game> Game {
             away_team_mvps: None,
         };
         let seed = game.get_rng_seed();
-        let mut rng = ChaCha8Rng::from_seed(seed);
+        let mut action_rng = ChaCha8Rng::from_seed(seed);
 
         let attendance = (BASE_ATTENDANCE as f32
             + (total_reputation.value() as f32).powf(2.0) * planet.total_population() as f32)
-            * rng.random_range(0.75..1.25)
+            * action_rng.random_range(0.75..1.25)
             * (1.0 + bonus_attendance);
         game.attendance = attendance as u32;
         let mut default_output = ActionOutput::default();
@@ -275,7 +281,7 @@ impl<'game> Game {
         game.attendance,
         if game.attendance == 69 { " (nice)" } else { "" }
     ),
-].choose(&mut rng).expect("There should be one option").clone();
+].choose(&mut action_rng).expect("There should be one option").clone();
 
         default_output.description = opening_text;
         default_output.random_seed = seed;
@@ -290,7 +296,7 @@ impl<'game> Game {
             self.away_team_in_game.stats.get(&player_id)?
         };
 
-        let best_stats = vec![
+        let best_stats = [
             ("Pts", stats.points, 100.0), //We want points to show as number 1
             (
                 "Reb",
@@ -302,11 +308,7 @@ impl<'game> Game {
             ("Ast", stats.assists, 2.0),
             (
                 "Brw",
-                if stats.brawls[0] > stats.brawls[1] {
-                    stats.brawls[0] - stats.brawls[1]
-                } else {
-                    stats.brawls[1] - stats.brawls[0]
-                },
+                stats.brawls[0].abs_diff(stats.brawls[1]),
                 if stats.brawls[0] > stats.brawls[1] {
                     3.0
                 } else {
@@ -323,7 +325,7 @@ impl<'game> Game {
 
         let score = best_stats
             .iter()
-            .map(|(_, s, m)| s.clone() as f32 * m.clone())
+            .map(|(_, s, m)| *s as f32 * *m)
             .sum::<f32>() as u32;
 
         let player = if let Some(p) = self.home_team_in_game.players.get(&player_id) {
@@ -338,13 +340,7 @@ impl<'game> Game {
             score,
             best_stats: best_stats
                 .iter()
-                .map(|(t, s, m)| {
-                    (
-                        t.to_string(),
-                        s.clone(),
-                        (s.clone() as f32 * m.clone()) as u32,
-                    )
-                })
+                .map(|(t, s, m)| (t.to_string(), *s, (*s as f32 * *m) as u32))
                 .sorted_by(|(_, _, a), (_, _, b)| b.cmp(a))
                 .take(3)
                 .collect_vec()
@@ -366,10 +362,8 @@ impl<'game> Game {
             .collect()
     }
 
-    fn pick_action(&self, rng: &mut ChaCha8Rng) -> Action {
-        let situation = self.action_results[self.action_results.len() - 1]
-            .situation
-            .clone();
+    fn pick_action(&self, action_rng: &mut ChaCha8Rng) -> Action {
+        let situation = self.action_results[self.action_results.len() - 1].situation;
 
         match situation {
             ActionSituation::JumpBall => Action::JumpBall,
@@ -383,17 +377,17 @@ impl<'game> Game {
                 let brawl_probability = BRAWL_ACTION_PROBABILITY
                     * (self.home_team_in_game.tactic.brawl_probability_modifier()
                         + self.away_team_in_game.tactic.brawl_probability_modifier());
-                if rng.random_bool(brawl_probability as f64) {
+                if action_rng.random_bool(brawl_probability as f64) {
                     Action::Brawl
                 } else {
                     match self.possession {
                         Possession::Home => self
                             .home_team_in_game
-                            .pick_action(rng)
+                            .pick_action(action_rng)
                             .unwrap_or(Action::Isolation),
                         Possession::Away => self
                             .away_team_in_game
-                            .pick_action(rng)
+                            .pick_action(action_rng)
                             .unwrap_or(Action::Isolation),
                     }
                 }
@@ -404,11 +398,11 @@ impl<'game> Game {
             | ActionSituation::Turnover => match self.possession {
                 Possession::Home => self
                     .home_team_in_game
-                    .pick_action(rng)
+                    .pick_action(action_rng)
                     .unwrap_or(Action::Isolation),
                 Possession::Away => self
                     .away_team_in_game
-                    .pick_action(rng)
+                    .pick_action(action_rng)
                     .unwrap_or(Action::Isolation),
             },
         }
@@ -447,8 +441,8 @@ impl<'game> Game {
 
         if let Some(updates) = &mut home_stats {
             for (id, player_stats) in self.home_team_in_game.stats.iter_mut() {
-                let player = self.home_team_in_game.players.get_mut(&id).unwrap();
-                if let Some(stats) = updates.get_mut(&id) {
+                let player = self.home_team_in_game.players.get_mut(id).unwrap();
+                if let Some(stats) = updates.get_mut(id) {
                     player_stats.update(stats);
                     player.add_tiredness(stats.extra_tiredness);
                     player.add_morale(stats.extra_morale);
@@ -473,9 +467,9 @@ impl<'game> Game {
         }
         if let Some(updates) = &mut away_stats {
             for (id, player_stats) in self.away_team_in_game.stats.iter_mut() {
-                let player = self.away_team_in_game.players.get_mut(&id).unwrap();
+                let player = self.away_team_in_game.players.get_mut(id).unwrap();
 
-                if let Some(stats) = updates.get_mut(&id) {
+                if let Some(stats) = updates.get_mut(id) {
                     player_stats.update(stats);
                     player.add_tiredness(stats.extra_tiredness);
                     player.add_morale(stats.extra_morale);
@@ -512,14 +506,14 @@ impl<'game> Game {
 
         if let Some(updates) = home_stats {
             for (id, player_stats) in self.home_team_in_game.stats.iter_mut() {
-                if let Some(update) = updates.get(&id) {
+                if let Some(update) = updates.get(id) {
                     player_stats.position = update.position;
                 }
             }
         }
         if let Some(updates) = away_stats {
             for (id, player_stats) in self.away_team_in_game.stats.iter_mut() {
-                if let Some(update) = updates.get(&id) {
+                if let Some(update) = updates.get(id) {
                     player_stats.position = update.position;
                 }
             }
@@ -531,7 +525,7 @@ impl<'game> Game {
     fn apply_tiredness_update(&mut self) {
         for team in [&mut self.home_team_in_game, &mut self.away_team_in_game] {
             for (id, player) in team.players.iter_mut() {
-                let stats = team.stats.get_mut(&id).expect("Player should have stats");
+                let stats = team.stats.get_mut(id).expect("Player should have stats");
                 if stats.is_playing() && !self.timer.is_break() {
                     stats.seconds_played += 1;
                     if !player.is_knocked_out() {
@@ -559,7 +553,7 @@ impl<'game> Game {
                 .by_position(&self.home_team_in_game.stats)
                 .iter()
                 .take(MAX_POSITION as usize)
-                .map(|&p| p)
+                .copied()
                 .collect::<Vec<&Player>>(),
             Possession::Away => self
                 .away_team_in_game
@@ -567,7 +561,7 @@ impl<'game> Game {
                 .by_position(&self.away_team_in_game.stats)
                 .iter()
                 .take(MAX_POSITION as usize)
-                .map(|&p| p)
+                .copied()
                 .collect::<Vec<&Player>>(),
         }
     }
@@ -580,7 +574,7 @@ impl<'game> Game {
                 .by_position(&self.away_team_in_game.stats)
                 .iter()
                 .take(5)
-                .map(|&p| p)
+                .copied()
                 .collect::<Vec<&Player>>(),
             Possession::Away => self
                 .home_team_in_game
@@ -588,7 +582,7 @@ impl<'game> Game {
                 .by_position(&self.home_team_in_game.stats)
                 .iter()
                 .take(5)
-                .map(|&p| p)
+                .copied()
                 .collect::<Vec<&Player>>(),
         }
     }
@@ -656,7 +650,7 @@ impl<'game> Game {
     fn game_end_description(&self, winner: Option<&str>) -> String {
         let (home, away) = self.get_score();
         if let Some(winner_name) = winner {
-            let loser_name = if winner_name.to_string() == self.home_team_in_game.name {
+            let loser_name = if winner_name == self.home_team_in_game.name {
                 self.away_team_in_game.name.clone()
             } else {
                 self.home_team_in_game.name.clone()
@@ -727,8 +721,12 @@ impl<'game> Game {
 
         self.apply_tiredness_update();
 
-        let seed = self.get_rng_seed();
-        let rng = &mut ChaCha8Rng::from_seed(seed);
+        let mut seed = self.get_rng_seed();
+        let action_rng = &mut ChaCha8Rng::from_seed(seed);
+
+        // Reverse seed just to get a different rng generator.
+        seed.reverse();
+        let description_rng = &mut ChaCha8Rng::from_seed(seed);
         let action_input = &self.action_results[self.action_results.len() - 1];
 
         if !self.timer.reached(self.next_step) {
@@ -737,16 +735,21 @@ impl<'game> Game {
 
         // If next tick is at a break, we are at the end of the quarter and should stop.
         if self.timer.is_break() {
-            if let Some(eoq) = EndOfQuarter::execute(action_input, self, rng) {
+            if let Some(eoq) =
+                EndOfQuarter::execute(action_input, self, action_rng, description_rng)
+            {
                 self.next_step = self.timer.period().next().start();
                 self.action_results.push(eoq);
                 return;
             }
         }
 
-        self.current_action = self.pick_action(rng);
+        self.current_action = self.pick_action(action_rng);
 
-        if let Some(mut result) = self.current_action.execute(action_input, self, rng) {
+        if let Some(mut result) =
+            self.current_action
+                .execute(action_input, self, action_rng, description_rng)
+        {
             self.apply_game_stats_update(
                 result.attack_stats_update.clone(),
                 result.defense_stats_update.clone(),
@@ -818,8 +821,7 @@ impl<'game> Game {
 
                         self.action_results.push(ActionOutput {
                             description: format!(
-                    "Both team are completely done! {} They should get some rest now...",
-                    description
+                    "Both team are completely done! {description} They should get some rest now..."
                 ),
                             start_at: self.timer,
                             end_at: self.timer,
@@ -871,7 +873,9 @@ impl<'game> Game {
                     (false, false) =>
                     // Check if teams make substitutions. Only if ball is out
                     {
-                        if let Some(sub) = Substitution::execute(action_input, self, rng) {
+                        if let Some(sub) =
+                            Substitution::execute(action_input, self, action_rng, description_rng)
+                        {
                             self.apply_sub_update(
                                 sub.attack_stats_update.clone(),
                                 sub.defense_stats_update.clone(),
@@ -888,46 +892,45 @@ impl<'game> Game {
 #[cfg(test)]
 mod tests {
     use super::Game;
-    use crate::game_engine::types::TeamInGame;
+    use crate::game_engine::action::Advantage;
+    use crate::game_engine::constants::NUMBER_OF_ROLLS;
+    use crate::game_engine::game::GameSummary;
+    use crate::game_engine::types::{GameStats, GameStatsMap, TeamInGame};
     use crate::types::{AppResult, GameId};
     use crate::types::{SystemTimeTick, Tick};
     use crate::world::constants::DEFAULT_PLANET_ID;
     use crate::world::world::World;
+    use itertools::Itertools;
     use rand::SeedableRng;
     use rand_chacha::ChaCha8Rng;
+    use std::collections::{BTreeMap, HashMap};
 
-    #[ignore]
     #[test]
     fn test_game() -> AppResult<()> {
         let mut world = World::new(None);
-        let rng = &mut ChaCha8Rng::seed_from_u64(world.seed);
-
+        let action_rng = &mut ChaCha8Rng::seed_from_u64(world.seed);
         let id0 = world.generate_random_team(
-            rng,
+            action_rng,
             DEFAULT_PLANET_ID.clone(),
             "Testen".to_string(),
             "Tosten".to_string(),
+            Some(0.0),
         )?;
         let id1 = world.generate_random_team(
-            rng,
+            action_rng,
             DEFAULT_PLANET_ID.clone(),
             "Holalo".to_string(),
             "Halley".to_string(),
+            Some(0.0),
         )?;
 
-        let home_team = world.get_team(&id0).unwrap().clone();
-
-        let checked_player_id = home_team.player_ids[0];
-        let quickness_before = world
-            .get_player_or_err(&checked_player_id)?
-            .athletics
-            .quickness
-            .clone();
-
         let home_team_in_game = TeamInGame::from_team_id(&id0, &world.teams, &world.players);
         let away_team_in_game = TeamInGame::from_team_id(&id1, &world.teams, &world.players);
 
-        let mut game = Game::new(
+        let home_rating = world.team_rating(&id0).unwrap_or_default();
+        let away_rating = world.team_rating(&id1).unwrap_or_default();
+
+        let game = Game::new(
             GameId::new_v4(),
             home_team_in_game.unwrap(),
             away_team_in_game.unwrap(),
@@ -935,38 +938,388 @@ mod tests {
             &world.get_planet(&DEFAULT_PLANET_ID).unwrap(),
         );
 
-        game.home_team_in_game
-            .players
-            .remove(&home_team.player_ids[1]);
-        game.home_team_in_game
-            .players
-            .remove(&home_team.player_ids[2]);
-        game.home_team_in_game
-            .players
-            .remove(&home_team.player_ids[3]);
-
-        game.home_team_in_game
-            .stats
-            .remove(&home_team.player_ids[1]);
-        game.home_team_in_game
-            .stats
-            .remove(&home_team.player_ids[2]);
-        game.home_team_in_game
-            .stats
-            .remove(&home_team.player_ids[3]);
-
-        println!("{:?}", game.home_team_in_game.players.len());
+        let game_id = game.id;
+        let home_tactic = game.home_team_in_game.tactic;
+        let away_tactic = game.away_team_in_game.tactic;
 
         world.games.insert(game.id, game);
+
+        // Call parts of the internal handle_tick loop by hand, to make the simulation go faster.
+        let mut game_action_results = vec![];
+        let mut game_stats = GameStatsMap::new();
         while world.games.len() > 0 {
-            let _ = world.handle_tick_events(Tick::now());
+            let current_tick = Tick::now();
+            for game in world.games.values_mut() {
+                if game.has_started(current_tick) && !game.has_ended() {
+                    game.tick(current_tick);
+                }
+
+                if game.has_ended() {
+                    let game_summary = GameSummary::from_game(&game);
+                    game_action_results = game.action_results.clone();
+                    game_stats = game.home_team_in_game.stats.clone();
+                    game_stats.extend(game.away_team_in_game.stats.clone());
+                    world.past_games.insert(game_summary.id, game_summary);
+                }
+            }
+
+            world.games.retain(|_, g| !g.has_ended());
+        }
+
+        let result = world.past_games.get(&game_id).unwrap();
+        let home_score: u16 = result.home_quarters_score.iter().sum();
+        let away_score: u16 = result.away_quarters_score.iter().sum();
+
+        println!(
+            "{:.2} vs {:.2} --> {}:{}\n{} -- {}",
+            home_rating, away_rating, home_score, away_score, home_tactic, away_tactic
+        );
+
+        let num_attack_advantages = game_action_results
+            .iter()
+            .filter(|a| a.advantage == Advantage::Attack)
+            .count();
+
+        let num_no_advantages = game_action_results
+            .iter()
+            .filter(|a| a.advantage == Advantage::Neutral)
+            .count();
+
+        let num_defense_advantages = game_action_results
+            .iter()
+            .filter(|a| a.advantage == Advantage::Defense)
+            .count();
+
+        println!(
+            "Advantages: {} {} {}",
+            num_attack_advantages, num_no_advantages, num_defense_advantages
+        );
+
+        let num_offensive_rebounds: u16 = game_stats
+            .iter()
+            .map(|(_, stat)| stat.offensive_rebounds)
+            .sum();
+
+        let num_defense_rebounds: u16 = game_stats
+            .iter()
+            .map(|(_, stat)| stat.defensive_rebounds)
+            .sum();
+        println!(
+            "Rebounds: off:{} def:{}",
+            num_offensive_rebounds, num_defense_rebounds,
+        );
+
+        let attempted_2pt: u16 = game_stats.iter().map(|(_, stat)| stat.attempted_2pt).sum();
+        let made_2pt: u16 = game_stats.iter().map(|(_, stat)| stat.made_2pt).sum();
+        let attempted_3pt: u16 = game_stats.iter().map(|(_, stat)| stat.attempted_3pt).sum();
+        let made_3pt: u16 = game_stats.iter().map(|(_, stat)| stat.made_3pt).sum();
+
+        println!(
+            "Shots: 2pt:{}/{} 3pt:{}/{}",
+            made_2pt, attempted_2pt, made_3pt, attempted_3pt
+        );
+
+        Ok(())
+    }
+
+    /// Sum the provided selector over a team's GameStatsMap.
+    /// `stats` is a GameStatsMap (player_id -> GameStats).
+    fn team_stat_sum<F>(stats: &GameStatsMap, selector: F) -> f32
+    where
+        F: Fn(&GameStats) -> f32,
+    {
+        stats.values().map(|stat| selector(stat)).sum()
+    }
+
+    fn get_simulated_game_stats(
+        world: &mut World,
+        action_rng: &mut ChaCha8Rng,
+        n_games: usize,
+    ) -> AppResult<Vec<(f32, GameStatsMap, GameStatsMap)>> {
+        let mut samples = Vec::with_capacity(n_games);
+        const DELTA: f32 = 6.0;
+        for i in 0..n_games {
+            let id0 = world.generate_random_team(
+                action_rng,
+                DEFAULT_PLANET_ID.clone(),
+                "Testen".to_string(),
+                "Tosten".to_string(),
+                Some(DELTA * i as f32 / n_games as f32),
+            )?;
+            let id1 = world.generate_random_team(
+                action_rng,
+                DEFAULT_PLANET_ID.clone(),
+                "Holalo".to_string(),
+                "Halley".to_string(),
+                Some(-DELTA * i as f32 / n_games as f32),
+            )?;
+
+            let home_team_in_game = TeamInGame::from_team_id(&id0, &world.teams, &world.players);
+            let away_team_in_game = TeamInGame::from_team_id(&id1, &world.teams, &world.players);
+
+            let home_rating = world.team_rating(&id0).unwrap_or_default();
+            let away_rating = world.team_rating(&id1).unwrap_or_default();
+
+            let mut game = Game::new(
+                GameId::new_v4(),
+                home_team_in_game.unwrap(),
+                away_team_in_game.unwrap(),
+                Tick::now(),
+                &world.get_planet(&DEFAULT_PLANET_ID).unwrap(),
+            );
+
+            // Simulate until finished
+            while !game.has_ended() {
+                let current_tick = Tick::now();
+                if game.has_started(current_tick) {
+                    game.tick(current_tick);
+                }
+            }
+
+            // Reorder so home team is always higher rated one.
+            if home_rating >= away_rating {
+                samples.push((
+                    home_rating - away_rating,
+                    game.home_team_in_game.stats.clone(),
+                    game.away_team_in_game.stats.clone(),
+                ));
+            } else {
+                samples.push((
+                    away_rating - home_rating,
+                    game.away_team_in_game.stats.clone(),
+                    game.home_team_in_game.stats.clone(),
+                ));
+            }
+        }
+
+        Ok(samples)
+    }
+
+    /// Returns (mean, stddev, count) per bin_center (int)
+    fn compute_binned_stats<F>(
+        samples: &Vec<(f32, GameStatsMap, GameStatsMap)>, // (rating_diff, home_stas, away_stats)
+        bin_size: f32,
+        selectors: Vec<F>,
+    ) -> BTreeMap<i32, ((Vec<f32>, Vec<f32>), (Vec<f32>, Vec<f32>), usize)>
+    where
+        F: Fn(&GameStats) -> f32,
+    {
+        // First pass: sum and count for each selector
+        let default_entry = (
+            vec![0.0f32].repeat(selectors.len()), // away avg/stddev for each selector
+            vec![0.0f32].repeat(selectors.len()), // home avg/stddev for each selector
+            0usize,
+        );
+        let mut sums: BTreeMap<i32, (Vec<f32>, Vec<f32>, usize)> = BTreeMap::new();
+        for (rating_diff, home_stats, away_stats) in samples {
+            let bin = ((*rating_diff) / bin_size).round() as i32;
+            let entry = sums.entry(bin).or_insert(default_entry.clone());
+            for (idx, selector) in selectors.iter().enumerate() {
+                entry.0[idx] += team_stat_sum(home_stats, selector);
+                entry.1[idx] += team_stat_sum(away_stats, selector);
+            }
+            entry.2 += 1;
+        }
+
+        // Means
+        let mut means: BTreeMap<i32, (Vec<f32>, Vec<f32>)> = BTreeMap::new();
+        for (bin, (home_sums, away_sums, count)) in &sums {
+            let home_means = (0..selectors.len())
+                .map(|idx| home_sums[idx] / *count as f32)
+                .collect();
+
+            let away_means = (0..selectors.len())
+                .map(|idx| away_sums[idx] / *count as f32)
+                .collect();
+
+            means.insert(*bin, (home_means, away_means));
+        }
+
+        // Second pass: sum squared deviations
+        let default_entry = (
+            vec![0.0f32].repeat(selectors.len()), // away avg/stddev for each selector
+            vec![0.0f32].repeat(selectors.len()), // home avg/stddev for each selector
+        );
+        let mut sqdevs: BTreeMap<i32, (Vec<f32>, Vec<f32>)> = BTreeMap::new();
+        for (rating_diff, home_stats, away_stats) in samples {
+            let bin = ((*rating_diff) / bin_size).round() as i32;
+            let (home_means, away_means) = means[&bin].clone();
+            let entry = sqdevs.entry(bin).or_insert(default_entry.clone());
+            for (idx, selector) in selectors.iter().enumerate() {
+                entry.0[idx] += (team_stat_sum(home_stats, selector) - home_means[idx]).powi(2);
+                entry.1[idx] += (team_stat_sum(away_stats, selector) - away_means[idx]).powi(2);
+            }
+        }
+
+        // Final assembly: compute sample variance (N-1), stddev, and count
+        let mut out = BTreeMap::new();
+        for (bin, (_, _, count)) in sums {
+            let (home_means, away_means) = means[&bin].clone();
+            let (home_ss, away_ss) = sqdevs.get(&bin).unwrap();
+            let home_variances = home_ss
+                .iter()
+                .map(|s| {
+                    if count > 1 {
+                        (s / (count as f32 - 1.0)).sqrt()
+                    } else {
+                        0.0
+                    }
+                })
+                .collect_vec();
+            let away_variances = away_ss
+                .iter()
+                .map(|s| {
+                    if count > 1 {
+                        (s / (count as f32 - 1.0)).sqrt()
+                    } else {
+                        0.0
+                    }
+                })
+                .collect_vec();
+
+            let bin_center = (bin as f32 * bin_size) as i32;
+            out.insert(
+                bin_center,
+                (
+                    (home_means, home_variances),
+                    (away_means, away_variances),
+                    count,
+                ),
+            );
+        }
+        out
+    }
+
+    #[ignore]
+    #[test]
+    fn test_multiple_games() -> AppResult<()> {
+        // Example usage: compute stat for made_2pt (you can change to any selector)
+        let mut world = World::new(None);
+        let mut action_rng = ChaCha8Rng::seed_from_u64(world.seed);
+
+        const N: usize = 10_000;
+        let samples = get_simulated_game_stats(&mut world, &mut action_rng, N)?;
+        let bin_size = 1.0;
+
+        let point_selector = |s: &GameStats| 2.0 * s.made_2pt as f32 + 3.0 * s.made_3pt as f32;
+        let win_samples = samples
+            .iter()
+            .filter(|(_, home_stats, away_stats)| {
+                team_stat_sum(home_stats, point_selector)
+                    > team_stat_sum(away_stats, point_selector)
+            })
+            .map(|(rating_diff, _, _)| ((*rating_diff) / bin_size).round() as i32)
+            .collect_vec();
+
+        let mut win_counts = HashMap::new();
+        for bin in win_samples {
+            *win_counts.entry(bin).or_insert(0) += 1;
+        }
+
+        println!("N={}", NUMBER_OF_ROLLS);
+
+        let selectors = vec![
+            |s: &GameStats| 2.0 * s.made_2pt as f32 + 3.0 * s.made_3pt as f32, // points
+            |s: &GameStats| s.made_2pt as f32,
+            |s: &GameStats| s.attempted_2pt as f32,
+            |s: &GameStats| s.made_3pt as f32,
+            |s: &GameStats| s.attempted_3pt as f32,
+            |s: &GameStats| s.defensive_rebounds as f32,
+            |s: &GameStats| s.offensive_rebounds as f32,
+            |s: &GameStats| s.assists as f32,
+            |s: &GameStats| s.turnovers as f32,
+            |s: &GameStats| s.steals as f32,
+            |s: &GameStats| s.blocks as f32,
+            |s: &GameStats| s.brawls[0] as f32 + 0.5 * s.brawls[1] as f32,
+        ];
+        let bin_stats = compute_binned_stats(&samples, bin_size, selectors);
+
+        for (bin_center, ((home_avg, home_stddev), (away_avg, away_stddev), count)) in bin_stats {
+            println!("Δrating={:+2} ({} samples)", bin_center, count);
+
+            let bin_win_counts = win_counts.get(&bin_center).copied().unwrap_or_default();
+
+            // The following formulas are not exact cause we consider draws a loss.
+            println!(
+                "  Win% = {:3.1} ± {:3.1} ({}/{})",
+                100.0 * (bin_win_counts + 1) as f32 / (count + 2) as f32,
+                100.0
+                    * (((bin_win_counts + 1) * (count - bin_win_counts + 1)) as f32
+                        / ((count + 2).pow(2) * (count + 3)) as f32)
+                        .sqrt(),
+                bin_win_counts,
+                count
+            );
+            println!(
+                "  points = {:3.1} ± {:3.1} vs {:3.1} ± {:3.1}",
+                home_avg[0], home_stddev[0], away_avg[0], away_stddev[0],
+            );
+            println!(
+                "  2pt = {:3.1}/{:3.1} ± {:3.1}/{:3.1} vs {:3.1}/{:3.1} ± {:3.1}/{:3.1}",
+                home_avg[1],
+                home_avg[2],
+                home_stddev[1],
+                home_stddev[2],
+                away_avg[1],
+                away_avg[2],
+                away_stddev[1],
+                away_stddev[2],
+            );
+            println!(
+                "  3pt = {:3.1}/{:3.1} ± {:3.1}/{:3.1} vs {:3.1}/{:3.1} ± {:3.1}/{:3.1}",
+                home_avg[3],
+                home_avg[4],
+                home_stddev[3],
+                home_stddev[4],
+                away_avg[3],
+                away_avg[4],
+                away_stddev[3],
+                away_stddev[4],
+            );
+
+            println!(
+                "  Def/Off Rebounds = {:3.1}/{:3.1} ± {:3.1}/{:3.1} vs {:3.1}/{:3.1} ± {:3.1}/{:3.1}",
+                home_avg[5],
+                home_avg[6],
+                home_stddev[5],
+                home_stddev[6],
+                away_avg[5],
+                away_avg[6],
+                away_stddev[5],
+                away_stddev[6],
+            );
+
+            println!(
+                "  Assists/Turnovers = {:3.1}/{:3.1} ± {:3.1}/{:3.1} vs {:3.1}/{:3.1} ± {:3.1}/{:3.1}",
+                home_avg[7],
+                home_avg[8],
+                home_stddev[7],
+                home_stddev[8],
+                away_avg[7],
+                away_avg[8],
+                away_stddev[7],
+                away_stddev[8],
+            );
+
+            println!(
+                "  Steals/Blocks = {:3.1}/{:3.1} ± {:3.1}/{:3.1} vs {:3.1}/{:3.1} ± {:3.1}/{:3.1}",
+                home_avg[9],
+                home_avg[10],
+                home_stddev[9],
+                home_stddev[10],
+                away_avg[9],
+                away_avg[10],
+                away_stddev[9],
+                away_stddev[10],
+            );
+
+            println!(
+                "  Brawls = {:3.1} ± {:3.1} vs {:3.1} ± {:3.1}",
+                home_avg[11], home_stddev[11], away_avg[11], away_stddev[11],
+            );
+
+            println!("");
         }
-        let quickness_after = world
-            .get_player_or_err(&checked_player_id)?
-            .athletics
-            .quickness
-            .clone();
-        println!("{} {}", quickness_before, quickness_after);
 
         Ok(())
     }
