@@ -16,12 +16,12 @@ use std::collections::HashMap;
 pub struct OffTheScreen;
 
 impl EngineAction for OffTheScreen {
-    fn execute(input: &ActionOutput, game: &Game, rng: &mut ChaCha8Rng) -> Option<ActionOutput> {
+    fn execute(input: &ActionOutput, game: &Game, action_rng: &mut ChaCha8Rng, description_rng: &mut ChaCha8Rng) -> Option<ActionOutput> {
         let attacking_players = game.attacking_players();
         let defending_players = game.defending_players();
 
         let play_idx = match input.attackers.len() {
-            0 => Self::sample(rng, [6, 1, 2, 0, 0])?,
+            0 => Self::sample(action_rng, [6, 1, 2, 0, 0])?,
             _ => input.attackers[0],
         };
         let target_idx = match input.attackers.len() {
@@ -29,7 +29,7 @@ impl EngineAction for OffTheScreen {
             _ => {
                 let mut weights = [1, 2, 3, 3, 2];
                 weights[play_idx] = 0;
-                Self::sample(rng, weights)?
+                Self::sample(action_rng, weights)?
             }
         };
 
@@ -47,19 +47,19 @@ impl EngineAction for OffTheScreen {
         let mut target_defender_update = GameStats::default();
         target_defender_update.extra_tiredness = TirednessCost::MEDIUM;
 
-        let timer_increase = 3 + rng.random_range(0..=1);
+        let timer_increase = 4 + action_rng.random_range(0..=2);
 
-        let atk_result = playmaker.roll(rng)
-            + playmaker.mental.vision.value()
-            + playmaker.technical.passing.value()
-            + target.mental.intuition.value();
+        let atk_result = playmaker.roll(action_rng)
+            + playmaker.mental.vision.game_value()
+            + playmaker.technical.passing.game_value()
+            + target.mental.intuition.game_value();
 
-        let def_result = playmaker_defender.roll(rng)
-            + target_defender.defense.perimeter_defense.value()
-            + target_defender.athletics.quickness.value();
+        let def_result = playmaker_defender.roll(action_rng)
+            + target_defender.defense.perimeter_defense.game_value()
+            + target_defender.athletics.quickness.game_value();
 
         let mut result = match atk_result as i16 - def_result as i16 + Self::tactic_modifier(game, &Action::OffTheScreen) {
-            x if x > ADV_ATTACK_LIMIT => ActionOutput {
+            x if x >= ADV_ATTACK_LIMIT => ActionOutput {
                 possession: input.possession,
                 advantage: Advantage::Attack,
                 attackers: vec![target_idx],
@@ -86,7 +86,7 @@ impl EngineAction for OffTheScreen {
                         "{} grabs the pass from {} and now has an easy opportunity for a shot.",
                         target.info.short_name(), playmaker.info.short_name(),
                     ),
-                ].choose(rng).expect("There should be one option").clone(),
+                ].choose(description_rng).expect("There should be one option").clone(),
                 assist_from: Some(play_idx),
                 start_at: input.end_at,
                 end_at: input.end_at.plus(timer_increase),
@@ -121,7 +121,7 @@ impl EngineAction for OffTheScreen {
                         "{} passes to {} as they come off the screen for a look at the basket.",
                         playmaker.info.short_name(), target.info.short_name(),
                     ),
-                ].choose(rng).expect("There should be one option").clone(),
+                ].choose(description_rng).expect("There should be one option").clone(),
                 assist_from: Some(play_idx),
                 start_at: input.end_at,
                 end_at: input.end_at.plus(timer_increase),
@@ -156,7 +156,7 @@ impl EngineAction for OffTheScreen {
                         "{} gets the pass after the screen, but {} doesn't give an inch, and the shot is off balance.",
                         playmaker.info.short_name(), target.info.short_name(),
                     ),
-                ].choose(rng).expect("There should be one option").clone(),
+                ].choose(description_rng).expect("There should be one option").clone(),
                 assist_from: Some(play_idx),
                 start_at: input.end_at,
                 end_at: input.end_at.plus(timer_increase),
@@ -166,15 +166,16 @@ impl EngineAction for OffTheScreen {
             },
             _ => {
                 playmaker_update.turnovers = 1;
-                target_defender_update.steals = 1;
                 playmaker_update.extra_morale += MoraleModifier::SMALL_MALUS;
                 target_defender_update.extra_morale += MoraleModifier::MEDIUM_BONUS;
 
+                let with_steal =  def_result >= 3* NUMBER_OF_ROLLS;
 
-                ActionOutput {
-                    situation: ActionSituation::Turnover,
-                    possession: !input.possession,
-                    description:[
+                if with_steal{
+                target_defender_update.steals = 1;
+                }
+
+                let description = if with_steal {[
                         format!(
                             "{} tries to pass to {} off-the-screen but {} blocks the pass.",
                             playmaker.info.short_name(), target.info.short_name(), target_defender.info.short_name()
@@ -190,14 +191,24 @@ impl EngineAction for OffTheScreen {
                         format!(
                             "{} tries to feed the ball to {} after the screen, but {} steals it away.",
                             playmaker.info.short_name(), target.info.short_name(), target_defender.info.short_name()
-                        ),
-                        format!(
+                        )
+                    ].choose(description_rng).expect("There should be one option").clone()} else {
+                        [format!(
                             "{} passes to {} off the screen, but the pass is too high and goes out of bounds.",
                             playmaker.info.short_name(), target.info.short_name()
-                        ),
-                    ].choose(rng).expect("There should be one option").clone(),
+                        ), 
+                        format!(
+                            "{} sends it to {}, but the pass is off the mark.",
+                            playmaker.info.short_name(), target.info.short_name()
+                        )].choose(description_rng).expect("There should be one option").clone()
+                    };
+
+                ActionOutput {
+                    situation: ActionSituation::Turnover,
+                    possession: !input.possession,
+                    description,
                     start_at: input.end_at,
-                end_at: input.end_at.plus(3),
+                end_at: input.end_at.plus(3 +  action_rng.random_range(0..=2)),
                 home_score: input.home_score,
                     away_score: input.away_score,
                     ..Default::default()
