@@ -18,19 +18,24 @@ use std::collections::HashMap;
 pub struct Post;
 
 impl EngineAction for Post {
-    fn execute(input: &ActionOutput, game: &Game, rng: &mut ChaCha8Rng) -> Option<ActionOutput> {
+    fn execute(
+        input: &ActionOutput,
+        game: &Game,
+        action_rng: &mut ChaCha8Rng,
+        description_rng: &mut ChaCha8Rng,
+    ) -> Option<ActionOutput> {
         let attacking_players = game.attacking_players();
         let defending_players = game.defending_players();
 
         let post_idx = match input.attackers.len() {
-            0 => Self::sample(rng, [0, 0, 1, 2, 3])?,
+            0 => Self::sample(action_rng, [0, 0, 1, 2, 3])?,
             _ => input.attackers[0],
         };
 
         let poster = attacking_players[post_idx];
         let defender = defending_players[post_idx];
 
-        let timer_increase = 4 + rng.random_range(0..=5);
+        let timer_increase = 5 + action_rng.random_range(0..=5);
 
         let mut attack_stats_update = HashMap::new();
         let mut post_update = GameStats::default();
@@ -40,18 +45,18 @@ impl EngineAction for Post {
         let mut defender_update = GameStats::default();
         defender_update.extra_tiredness = TirednessCost::MEDIUM;
 
-        let atk_result = poster.roll(rng)
-            + poster.technical.post_moves.value()
-            + poster.athletics.strength.value();
+        let atk_result = poster.roll(action_rng)
+            + poster.technical.post_moves.game_value()
+            + poster.athletics.strength.game_value();
 
-        let def_result = defender.roll(rng)
-            + defender.defense.interior_defense.value()
-            + defender.athletics.strength.value();
+        let def_result = defender.roll(action_rng)
+            + defender.defense.interior_defense.game_value()
+            + defender.athletics.strength.game_value();
 
         let mut result = match atk_result as i16 - def_result as i16
             + Self::tactic_modifier(game, &Action::Post)
         {
-            x if x > ADV_ATTACK_LIMIT => ActionOutput {
+            x if x >= ADV_ATTACK_LIMIT => ActionOutput {
                 possession: input.possession,
                 advantage: Advantage::Attack,
                 attackers: vec![post_idx],
@@ -84,7 +89,7 @@ impl EngineAction for Post {
                         defender.info.short_name()
                     ),
                 ]
-                .choose(rng)
+                .choose(description_rng)
                 .expect("There should be one option")
                 .clone(),
                 start_at: input.end_at,
@@ -126,7 +131,7 @@ impl EngineAction for Post {
                         defender.info.short_name(),
                     ),
                 ]
-                .choose(rng)
+                .choose(description_rng)
                 .expect("There should be one option")
                 .clone(),
                 start_at: input.end_at,
@@ -139,7 +144,7 @@ impl EngineAction for Post {
                 if poster.mental.vision as i16 + x > ADV_NEUTRAL_LIMIT {
                     let mut weights = [3, 3, 2, 2, 1];
                     weights[post_idx] = 0;
-                    let target_idx = WeightedIndex::new(&weights).ok()?.sample(rng);
+                    let target_idx = WeightedIndex::new(weights).ok()?.sample(action_rng);
                     let target: &Player = attacking_players[target_idx];
                     ActionOutput {
                         possession: input.possession,
@@ -168,7 +173,7 @@ impl EngineAction for Post {
                                 "{} can't find an opening against {}'s defense, so the ball is passed out to {} to reset.",
                                 poster.info.short_name(), defender.info.short_name(), target.info.short_name()
                             ),
-                        ].choose(rng)
+                        ].choose(description_rng)
                         .expect("There should be one option")
                         .clone(),
                         start_at: input.end_at,
@@ -205,7 +210,7 @@ impl EngineAction for Post {
                                 "{} makes an attempt in the post against {} but is completely shut down, forcing a bad shot.",
                                 poster.info.short_name(), defender.info.short_name()
                             ),
-                        ].choose(rng)
+                        ].choose(description_rng)
                         .expect("There should be one option")
                         .clone(),
                         start_at: input.end_at,
@@ -219,19 +224,45 @@ impl EngineAction for Post {
             _ => {
                 post_update.turnovers = 1;
                 post_update.extra_morale += MoraleModifier::SMALL_MALUS;
-                defender_update.steals = 1;
                 defender_update.extra_morale += MoraleModifier::MEDIUM_BONUS;
 
+                let with_steal = def_result >= 3 * NUMBER_OF_ROLLS;
+
+                if with_steal {
+                    defender_update.steals = 1;
+                }
+
+                let description = if with_steal {
+                    [
+                        format!(
+                            "{} steals the ball from {} on the post.",
+                            defender.info.short_name(),
+                            poster.info.short_name(),
+                        ),
+                        format!(
+                            "{} stops {} on the post and snaps the ball from {} hands.",
+                            defender.info.short_name(),
+                            poster.info.short_name(),
+                            poster.info.pronouns.as_possessive()
+                        ),
+                    ]
+                    .choose(description_rng)
+                    .expect("There should be one option")
+                    .clone()
+                } else {
+                    format!(
+                        "{}'s good defense is too much for {}, who fumbles the ball.",
+                        defender.info.short_name(),
+                        poster.info.short_name(),
+                    )
+                };
+
                 ActionOutput {
                     situation: ActionSituation::Turnover,
                     possession: !input.possession,
-                    description: format!(
-                        "{} steals the ball from {} on the post.",
-                        defender.info.short_name(),
-                        poster.info.short_name(),
-                    ),
+                    description,
                     start_at: input.end_at,
-                    end_at: input.end_at.plus(3),
+                    end_at: input.end_at.plus(5 + action_rng.random_range(0..=6)),
                     home_score: input.home_score,
                     away_score: input.away_score,
                     ..Default::default()
