@@ -19,7 +19,7 @@ impl PrintableGif for Gif {
         decoder.set_color_output(gif::ColorOutput::RGBA);
         let file = ASSETS_DIR
             .get_file(filename.clone())
-            .ok_or(anyhow!("Unable to open file {}", filename))?
+            .ok_or(anyhow!("Unable to open file {filename}"))?
             .contents();
         let mut decoder = decoder.read_info(file)?;
         let mut gif: Gif = vec![];
@@ -29,13 +29,13 @@ impl PrintableGif for Gif {
                 frame.height as u32,
                 frame.buffer.to_vec(),
             )
-            .ok_or(anyhow!("Unable to decode file {} into gif", filename))?;
+            .ok_or(anyhow!("Unable to decode file {filename} into gif"))?;
             gif.push(img);
         }
         Ok(gif)
     }
 
     fn to_lines(&self) -> GifLines {
-        self.iter().map(|img| img_to_lines(img)).collect()
+        self.iter().map(img_to_lines).collect()
     }
 }
