@@ -5,8 +5,8 @@ use crate::game_engine::types::TeamInGame;
 use crate::store::deserialize;
 use crate::types::{AppResult, SystemTimeTick, TeamId, Tick};
 use crate::ui::popup_message::PopupMessage;
-use crate::ui::utils::SwarmPanelEvent;
 use crate::world::constants::NETWORK_GAME_START_DELAY;
+use crate::world::MAX_AVG_TIREDNESS_PER_AUTO_GAME;
 use crate::{app::App, types::AppCallback};
 use anyhow::anyhow;
 use libp2p::gossipsub::TopicHash;
@@ -50,12 +50,8 @@ pub enum NetworkCallback {
 impl NetworkCallback {
     fn push_swarm_panel_message(timestamp: Tick, peer_id: PeerId, text: String) -> AppCallback {
         Box::new(move |app: &mut App| {
-            let event = SwarmPanelEvent {
-                timestamp,
-                peer_id: Some(peer_id),
-                text: text.clone(),
-            };
-            app.ui.swarm_panel.push_chat_event(event);
+            app.ui
+                .push_chat_event(timestamp, Some(peer_id), text.clone());
             Ok(None)
         })
     }
@@ -63,15 +59,9 @@ impl NetworkCallback {
     fn bind_address(address: Multiaddr) -> AppCallback {
         Box::new(move |app: &mut App| {
             app.ui
-                .push_log_event(Tick::now(), None, format!("Bound to {}", address));
+                .push_log_event(Tick::now(), None, format!("Bound to {address}"));
 
-            let network_handler = app
-                .network_handler
-                .as_mut()
-                .expect("Should have network handler");
-            network_handler.address = address.clone();
-
-            network_handler.dial_seed()?;
+            app.network_handler.dial_seed()?;
 
             Ok(None)
         })
@@ -80,7 +70,7 @@ impl NetworkCallback {
     fn subscribe(topic: TopicHash) -> AppCallback {
         Box::new(move |app: &mut App| {
             app.ui
-                .push_log_event(Tick::now(), None, format!("Subscribed to topic: {}", topic));
+                .push_log_event(Tick::now(), None, format!("Subscribed to topic: {topic}"));
             app.world.dirty_network = true;
             Ok(None)
         })
@@ -101,21 +91,15 @@ impl NetworkCallback {
 
     fn close_connection(peer_id: PeerId) -> AppCallback {
         Box::new(move |app: &mut App| {
-            if !app
-                .network_handler
-                .as_ref()
-                .expect("Should have network handler")
-                .swarm
-                .is_connected(&peer_id)
-            {
-                app.ui.push_log_event(
-                    Tick::now(),
-                    Some(peer_id),
-                    format!("Closing connection: {}", peer_id),
-                );
-                // FIXME: read connection protocol and understand when this is called.
-                //        For example, we could check that num_established >0 or that cause = None
-            }
+            // if !app.network_handler.swarm.is_connected(&peer_id) {
+            app.ui.push_log_event(
+                Tick::now(),
+                Some(peer_id),
+                format!("Closing connection: {peer_id}"),
+            );
+            // FIXME: read connection protocol and understand when this is called.
+            //        For example, we could check that num_established >0 or that cause = None
+            // }
             Ok(None)
         })
     }
@@ -129,7 +113,7 @@ impl NetworkCallback {
             app.ui.push_log_event(
                 Tick::now(),
                 peer_id,
-                format!("Got a team from peer: {:?}", peer_id),
+                format!("Got a team from peer: {peer_id:?}"),
             );
 
             if let Some(id) = peer_id {
@@ -151,12 +135,7 @@ impl NetworkCallback {
 
     fn handle_message_topic(peer_id: Option<PeerId>, timestamp: Tick, text: String) -> AppCallback {
         Box::new(move |app: &mut App| {
-            let event = SwarmPanelEvent {
-                timestamp,
-                peer_id,
-                text: text.clone(),
-            };
-            app.ui.swarm_panel.push_chat_event(event);
+            app.ui.push_chat_event(timestamp, peer_id, text.clone());
             Ok(None)
         })
     }
@@ -216,31 +195,8 @@ impl NetworkCallback {
                 });
             }
 
-            let own_version_major = env!("CARGO_PKG_VERSION_MAJOR").parse()?;
-            let own_version_minor = env!("CARGO_PKG_VERSION_MINOR").parse()?;
-            let own_version_patch = env!("CARGO_PKG_VERSION_PATCH").parse()?;
-
             // Notify about new version (only once).
-            if app.new_version_notified == false {
-                if seed_info.version_major > own_version_major
-                    || (seed_info.version_major == own_version_major
-                        && seed_info.version_minor > own_version_minor)
-                    || (seed_info.version_major == own_version_major
-                        && seed_info.version_minor == own_version_minor
-                        && seed_info.version_patch > own_version_patch)
-                {
-                    let message = format!(
-                        "New version {}.{}.{} available. Download at https://rebels.frittura.org",
-                        seed_info.version_major, seed_info.version_minor, seed_info.version_patch,
-                    );
-                    app.ui.push_popup(PopupMessage::Ok {
-                        message,
-                        is_skippable: false,
-                        tick: timestamp,
-                    });
-                    app.new_version_notified = true;
-                }
-            }
+            app.notify_seed_version(seed_info.version)?;
 
             app.ui
                 .swarm_panel
@@ -267,12 +223,7 @@ impl NetworkCallback {
                 ),
             );
 
-            let network_handler = app
-                .network_handler
-                .as_mut()
-                .expect("There should be a network handler");
-
-            let self_peer_id = network_handler.swarm.local_peer_id();
+            let self_peer_id = app.network_handler.own_peer_id();
             match &trade.state {
                 NetworkRequestState::Syn => {
                     if trade.proposer_peer_id == *self_peer_id {
@@ -340,16 +291,16 @@ impl NetworkCallback {
                         app.ui.push_log_event(
                             timestamp,
                             peer_id,
-                            format!("Trade accepted, players swapped"),
+                            "Trade accepted, players swapped".to_string(),
                         );
 
                         app.ui.push_popup(PopupMessage::Ok {
-                            message: format!("Trade accepted, players swapped."),
+                            message: "Trade accepted, players swapped.".to_string(),
                             is_skippable: false,
                             tick: Tick::now(),
                         });
                         trade.state = NetworkRequestState::Ack;
-                        network_handler.send_trade(trade)?;
+                        app.network_handler.send_trade(trade)?;
                         Ok(())
                     };
 
@@ -360,7 +311,7 @@ impl NetworkCallback {
                         };
                         let own_team = app.world.get_own_team_mut()?;
                         own_team.remove_trade(trade.proposer_player.id, trade.target_player.id);
-                        network_handler.send_trade(trade)?;
+                        app.network_handler.send_trade(trade)?;
 
                         return Err(anyhow!(err.to_string()));
                     }
@@ -373,7 +324,7 @@ impl NetworkCallback {
                         app.ui.push_log_event(
                             timestamp,
                             peer_id,
-                            format!("A trade is happening in the network"),
+                            "A trade is happening in the network".to_string(),
                         );
 
                         return Ok(None);
@@ -425,11 +376,11 @@ impl NetworkCallback {
                         app.ui.push_log_event(
                             timestamp,
                             peer_id,
-                            format!("Trade accepted, players swapped"),
+                            "Trade accepted, players swapped".to_string(),
                         );
 
                         app.ui.push_popup(PopupMessage::Ok {
-                            message: format!("Trade accepted, players swapped."),
+                            message: "Trade accepted, players swapped.".to_string(),
                             is_skippable: false,
                             tick: Tick::now(),
                         });
@@ -441,7 +392,7 @@ impl NetworkCallback {
                         trade.state = NetworkRequestState::Failed {
                             error_message: err.to_string(),
                         };
-                        network_handler.send_trade(trade)?;
+                        app.network_handler.send_trade(trade)?;
                         return Err(anyhow!(err.to_string()));
                     }
                 }
@@ -457,11 +408,11 @@ impl NetworkCallback {
                     own_team.remove_trade(trade.proposer_player.id, trade.target_player.id);
 
                     app.ui.push_popup(PopupMessage::Error {
-                        message: format!("Trade failed: {}", error_message),
+                        message: format!("Trade failed: {error_message}"),
                         tick: Tick::now(),
                     });
 
-                    return Err(anyhow!(format!("Trade failed: {}", error_message)))?;
+                    return Err(anyhow!(format!("Trade failed: {error_message}")))?;
                 }
             }
 
@@ -475,18 +426,13 @@ impl NetworkCallback {
         challenge: Challenge,
     ) -> AppCallback {
         Box::new(move |app: &mut App| {
-            let network_handler = app
-                .network_handler
-                .as_mut()
-                .expect("There should be a network handler");
-
             app.ui.push_log_event(
                 timestamp,
                 peer_id,
                 format!("\nChallenge: {}", challenge.format()),
             );
 
-            let self_peer_id = network_handler.swarm.local_peer_id();
+            let self_peer_id = app.network_handler.own_peer_id();
             match &challenge.state {
                 NetworkRequestState::Syn => {
                     if challenge.proposer_peer_id == *self_peer_id {
@@ -497,12 +443,31 @@ impl NetworkCallback {
                         return Err(anyhow!("Team is not challenge receiver"));
                     }
 
+                    let [own_major_version, own_minor_version, own_patch_version] =
+                        app.app_version();
+                    let [challenge_major_version, challenge_minor_version, challenge_patch_version] =
+                        challenge.app_version;
+                    if challenge_major_version != own_major_version
+                        || challenge_minor_version != own_minor_version
+                    {
+                        return Err(anyhow!(
+                            "App versions do not match: Proposer version {challenge_major_version}.{challenge_minor_version}.{challenge_patch_version} - Target version {own_major_version}.{own_minor_version}.{own_patch_version}"
+                        ));
+                    }
+
+                    let own_team = app.world.get_own_team()?;
+                    let average_tiredness = own_team.average_tiredness(&app.world);
+
                     let own_team = app.world.get_own_team_mut()?;
 
-                    if own_team.autonomous_strategy.challenge_network {
+                    if own_team.current_game.is_none()
+                        && own_team.autonomous_strategy.challenge_network
+                        && average_tiredness <= MAX_AVG_TIREDNESS_PER_AUTO_GAME
+                    {
                         let rng = &mut ChaCha8Rng::from_os_rng();
                         own_team.player_ids.shuffle(rng);
-                        network_handler.accept_challenge(&app.world, challenge.clone())?;
+                        app.network_handler
+                            .accept_challenge(&app.world, challenge.clone())?;
                         return Ok(Some("Challenge received.\nAuto accepted".to_string()));
                     }
 
@@ -523,6 +488,19 @@ impl NetworkCallback {
                     if challenge.proposer_peer_id != *self_peer_id {
                         return Err(anyhow!("Invalid challenge: team is not challenge sender"));
                     }
+
+                    let [own_major_version, own_minor_version, own_patch_version] =
+                        app.app_version();
+                    let [challenge_major_version, challenge_minor_version, challenge_patch_version] =
+                        challenge.app_version;
+                    if challenge_major_version != own_major_version
+                        || challenge_minor_version != own_minor_version
+                    {
+                        return Err(anyhow!(
+                            "App versions do not match: Proposer version {challenge_major_version}.{challenge_minor_version}.{challenge_patch_version} - Target version {own_major_version}.{own_minor_version}.{own_patch_version}"
+                        ));
+                    }
+
                     let mut handle_syn_ack = || -> AppResult<()> {
                         let mut home_team_in_game = TeamInGame::from_team_id(
                             &app.world.own_team_id,
@@ -531,8 +509,7 @@ impl NetworkCallback {
                         )
                         .ok_or(anyhow!("Cannot generate team in game"))?;
 
-                        home_team_in_game.peer_id =
-                            Some(network_handler.swarm.local_peer_id().clone());
+                        home_team_in_game.peer_id = Some(*app.network_handler.own_peer_id());
 
                         let mut challenge = challenge.clone();
                         challenge.home_team_in_game = home_team_in_game;
@@ -543,7 +520,7 @@ impl NetworkCallback {
                         app.ui.push_log_event(
                             timestamp,
                             peer_id,
-                            format!("Challenge accepted, generating game"),
+                            "Challenge accepted, generating game".to_string(),
                         );
 
                         let own_team = app.world.get_own_team_mut()?;
@@ -560,18 +537,18 @@ impl NetworkCallback {
                             challenge.state = NetworkRequestState::Failed {
                                 error_message: err.to_string(),
                             };
-                            network_handler.send_challenge(challenge)?;
+                            app.network_handler.send_challenge(challenge)?;
 
                             return Err(anyhow!(err.to_string()));
                         }
 
                         app.ui.push_popup(PopupMessage::Ok {
-                            message: format!("Challenge accepted, game is starting."),
+                            message: "Challenge accepted, game is starting.".to_string(),
                             is_skippable: false,
                             tick: Tick::now(),
                         });
 
-                        network_handler.send_challenge(challenge)?;
+                        app.network_handler.send_challenge(challenge)?;
                         Ok(())
                     };
 
@@ -581,7 +558,7 @@ impl NetworkCallback {
                         challenge.state = NetworkRequestState::Failed {
                             error_message: err.to_string(),
                         };
-                        network_handler.send_challenge(challenge)?;
+                        app.network_handler.send_challenge(challenge)?;
                         return Err(anyhow!(err.to_string()));
                     }
                 }
@@ -594,7 +571,7 @@ impl NetworkCallback {
                         app.ui.push_log_event(
                             timestamp,
                             peer_id,
-                            format!("Adding challenge from network"),
+                            "Adding challenge from network".to_string(),
                         );
 
                         if let Some(starting_at) = challenge.starting_at {
@@ -622,7 +599,7 @@ impl NetworkCallback {
                         app.ui.push_log_event(
                             timestamp,
                             peer_id,
-                            format!("Challenge accepted, generating game"),
+                            "Challenge accepted, generating game".to_string(),
                         );
 
                         if let Some(starting_at) = challenge.starting_at {
@@ -642,7 +619,7 @@ impl NetworkCallback {
                         }
 
                         app.ui.push_popup(PopupMessage::Ok {
-                            message: format!("Challenge accepted, game is starting."),
+                            message: "Challenge accepted, game is starting.".to_string(),
                             is_skippable: false,
                             tick: Tick::now(),
                         });
@@ -654,9 +631,9 @@ impl NetworkCallback {
                         challenge.state = NetworkRequestState::Failed {
                             error_message: err.to_string(),
                         };
-                        network_handler.send_challenge(challenge)?;
+                        app.network_handler.send_challenge(challenge)?;
                         app.ui.push_popup(PopupMessage::Error {
-                            message: format!("Challenge failed: {}", err),
+                            message: format!("Challenge failed: {err}"),
                             tick: Tick::now(),
                         });
 
@@ -678,11 +655,11 @@ impl NetworkCallback {
                     );
 
                     app.ui.push_popup(PopupMessage::Error {
-                        message: format!("Challenge failed: {}", error_message),
+                        message: format!("Challenge failed: {error_message}"),
                         tick: Tick::now(),
                     });
 
-                    return Err(anyhow!("Challenge failed: {}", error_message))?;
+                    return Err(anyhow!("Challenge failed: {error_message}"))?;
                 }
             }
 
@@ -696,30 +673,22 @@ impl NetworkCallback {
                 timestamp,
                 peer_id,
                 text,
-            } => Self::push_swarm_panel_message(timestamp.clone(), peer_id.clone(), text.clone())(
-                app,
-            ),
+            } => Self::push_swarm_panel_message(*timestamp, *peer_id, text.clone())(app),
             Self::PushSwarmPanelLog { timestamp, text } => {
                 app.ui.push_log_event(*timestamp, None, text.clone());
                 Ok(None)
             }
             Self::BindAddress { address } => Self::bind_address(address.clone())(app),
             Self::Subscribe { peer_id: _, topic } => Self::subscribe(topic.clone())(app),
-            Self::Unsubscribe { peer_id, topic } => {
-                Self::unsubscribe(peer_id.clone(), topic.clone())(app)
-            }
-            Self::CloseConnection { peer_id } => Self::close_connection(peer_id.clone())(app),
+            Self::Unsubscribe { peer_id, topic } => Self::unsubscribe(*peer_id, topic.clone())(app),
+            Self::CloseConnection { peer_id } => Self::close_connection(*peer_id)(app),
             Self::HandleConnectionEstablished { peer_id } => {
-                let network_handler = app
-                    .network_handler
-                    .as_mut()
-                    .expect("Should have network handler");
-                network_handler.send_own_team(&app.world)?;
+                app.network_handler.send_own_team(&app.world)?;
 
                 app.ui.push_log_event(
                     Tick::now(),
-                    Some(peer_id.clone()),
-                    format!("Connected to peer: {}", peer_id),
+                    Some(*peer_id),
+                    format!("Connected to peer: {peer_id}"),
                 );
                 Ok(None)
             }
