@@ -1,20 +1,21 @@
+use crate::app::AppEvent;
 use crate::network::constants::{DEFAULT_SEED_PORT, TOPIC};
 use crate::network::types::{NetworkData, PlayerRanking, TeamRanking};
 use crate::network::{handler::NetworkHandler, types::SeedInfo};
 use crate::store::*;
 use crate::types::{AppResult, PlayerId, TeamId};
-use futures::StreamExt;
 use itertools::Itertools;
 use libp2p::gossipsub::IdentTopic;
 use libp2p::{gossipsub, swarm::SwarmEvent};
 use std::collections::HashMap;
-use tokio::select;
+use tokio::sync::mpsc;
+use tokio_util::sync::CancellationToken;
 
 const TOP_PLAYER_RANKING_LENGTH: usize = 20;
 const TOP_TEAM_RANKING_LENGTH: usize = 10;
 
 pub struct Relayer {
-    pub running: bool,
+    running: bool,
     network_handler: NetworkHandler,
     team_ranking: HashMap<TeamId, TeamRanking>,
     top_team_ranking: Vec<(TeamId, TeamRanking)>,
@@ -24,6 +25,12 @@ pub struct Relayer {
     last_message_sent_to_team: HashMap<TeamId, usize>,
 }
 
+impl Default for Relayer {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl Relayer {
     fn get_top_player_ranking(&self) -> Vec<(PlayerId, PlayerRanking)> {
         self.player_ranking
@@ -35,7 +42,7 @@ impl Relayer {
                     .expect("Reputation should exist")
             })
             .take(TOP_PLAYER_RANKING_LENGTH)
-            .map(|(id, ranking)| (id.clone(), ranking.clone()))
+            .map(|(id, ranking)| (*id, ranking.clone()))
             .collect()
     }
 
@@ -49,7 +56,7 @@ impl Relayer {
                     .expect("Reputation should exist")
             })
             .take(TOP_TEAM_RANKING_LENGTH)
-            .map(|(id, ranking)| (id.clone(), ranking.clone()))
+            .map(|(id, ranking)| (*id, ranking.clone()))
             .collect()
     }
 
@@ -73,7 +80,7 @@ impl Relayer {
                     .expect("Reputation should exist")
             })
             .take(TOP_TEAM_RANKING_LENGTH)
-            .map(|(id, ranking)| (id.clone(), ranking.clone()))
+            .map(|(id, ranking)| (*id, ranking.clone()))
             .collect();
 
         let player_ranking = match load_player_ranking() {
@@ -95,12 +102,12 @@ impl Relayer {
                     .expect("Reputation should exist")
             })
             .take(TOP_PLAYER_RANKING_LENGTH)
-            .map(|(id, ranking)| (id.clone(), ranking.clone()))
+            .map(|(id, ranking)| (*id, ranking.clone()))
             .collect();
 
         Self {
             running: true,
-            network_handler: NetworkHandler::new(None, DEFAULT_SEED_PORT)
+            network_handler: NetworkHandler::new(None)
                 .expect("Failed to initialize network handler"),
             team_ranking,
             top_team_ranking,
@@ -113,16 +120,24 @@ impl Relayer {
 
     pub async fn run(&mut self) -> AppResult<()> {
         println!("Starting relayer. Press Ctrl-C to exit.");
+        let (event_sender, mut event_receiver) = mpsc::channel(256);
+
+        let cancellation_token = CancellationToken::new();
+        self.network_handler.start_polling_events(
+            event_sender,
+            cancellation_token.clone(),
+            DEFAULT_SEED_PORT,
+        );
         while self.running {
-            select! {
-                    swarm_event = self.network_handler.swarm.select_next_some() =>  {
-                        let result = self.handle_network_events(swarm_event);
-                        if result.is_err() {
-                            log::error!("Error handling network event: {:?}", result);
-                        }
+            if let Some(AppEvent::NetworkEvent(swarm_event)) = event_receiver.recv().await {
+                let result = self.handle_network_events(swarm_event);
+                if result.is_err() {
+                    log::error!("Error handling network event: {result:?}");
                 }
             }
         }
+
+        cancellation_token.cancel();
         Ok(())
     }
 
@@ -130,14 +145,14 @@ impl Relayer {
         &mut self,
         network_event: SwarmEvent<gossipsub::Event>,
     ) -> AppResult<()> {
-        println!("Received network event: {:?}", network_event);
+        println!("Received network event: {network_event:?}");
         match network_event {
             SwarmEvent::Behaviour(gossipsub::Event::Subscribed { peer_id, topic }) => {
                 if topic == IdentTopic::new(TOPIC).hash() {
-                    println!("Sending info to {}", peer_id);
+                    println!("Sending info to {peer_id}");
 
                     self.network_handler.send_seed_info(SeedInfo::new(
-                        self.network_handler.swarm.connected_peers().count(),
+                        self.network_handler.connected_peers_count,
                         None,
                         self.top_team_ranking.clone(),
                         self.top_player_ranking.clone(),
@@ -148,79 +163,72 @@ impl Relayer {
             SwarmEvent::Behaviour(gossipsub::Event::Message { message, .. }) => {
                 assert!(message.topic == IdentTopic::new(TOPIC).hash());
                 let network_data = deserialize::<NetworkData>(&message.data)?;
-                match network_data {
-                    NetworkData::Team(timestamp, network_team) => {
-                        if let Some(current_ranking) = self.team_ranking.get(&network_team.team.id)
-                        {
-                            if current_ranking.timestamp >= timestamp {
-                                return Ok(());
-                            }
-                        } else {
-                            self.network_handler.send_seed_info(SeedInfo::new(
-                                self.network_handler.swarm.connected_peers().count(),
-                                Some(format!(
-                                    "A new crew has started roaming the galaxy: {}",
-                                    network_team.team.name
-                                )),
-                                self.top_team_ranking.clone(),
-                                self.top_player_ranking.clone(),
-                            )?)?;
+                if let NetworkData::Team(timestamp, network_team) = network_data {
+                    if let Some(current_ranking) = self.team_ranking.get(&network_team.team.id) {
+                        if current_ranking.timestamp >= timestamp {
+                            return Ok(());
                         }
-
-                        let ranking = TeamRanking::from_network_team(timestamp, &network_team);
-
-                        // If the team is already stored, remove players from previous version.
-                        // This is to ensure that fired players are removed.
-                        if let Some(current_ranking) = self.team_ranking.get(&network_team.team.id)
-                        {
-                            for player_id in current_ranking.team.player_ids.iter() {
-                                self.player_ranking.remove(player_id);
-                            }
-                        }
-
-                        self.team_ranking
-                            .insert(network_team.team.id, ranking.clone());
-
-                        if let Err(err) = save_team_ranking(&self.team_ranking, true) {
-                            println!("Error while saving team ranking: {err}");
-                        }
-
-                        for player in network_team.players.iter() {
-                            let ranking = PlayerRanking::new(timestamp, player.clone());
-                            self.player_ranking.insert(player.id, ranking.clone());
-                        }
-
-                        if let Err(err) = save_player_ranking(&self.player_ranking, true) {
-                            println!("Error while saving player ranking: {err}");
-                        }
-
-                        self.top_team_ranking = self.get_top_team_ranking();
-                        self.top_player_ranking = self.get_top_player_ranking();
-
-                        // Check if there are new messages to send and append them to self.messages.
-                        self.messages.extend(load_relayer_messages()?);
-
-                        // Send messages starting from last sent message.
-                        let last_message_sent = self
-                            .last_message_sent_to_team
-                            .get(&network_team.team.id)
-                            .unwrap_or(&0);
-
-                        for (index, message) in self.messages.iter().enumerate() {
-                            if index < *last_message_sent {
-                                continue;
-                            }
-
-                            self.network_handler.send_relayer_message_to_team(
-                                message.clone(),
-                                network_team.team.id,
-                            )?;
-                        }
-
-                        self.last_message_sent_to_team
-                            .insert(network_team.team.id, self.messages.len());
+                    } else {
+                        self.network_handler.send_seed_info(SeedInfo::new(
+                            self.network_handler.connected_peers_count,
+                            Some(format!(
+                                "A new crew has started roaming the galaxy: {}",
+                                network_team.team.name
+                            )),
+                            self.top_team_ranking.clone(),
+                            self.top_player_ranking.clone(),
+                        )?)?;
                     }
-                    _ => {}
+
+                    let ranking = TeamRanking::from_network_team(timestamp, &network_team);
+
+                    // If the team is already stored, remove players from previous version.
+                    // This is to ensure that fired players are removed.
+                    if let Some(current_ranking) = self.team_ranking.get(&network_team.team.id) {
+                        for player_id in current_ranking.team.player_ids.iter() {
+                            self.player_ranking.remove(player_id);
+                        }
+                    }
+
+                    self.team_ranking
+                        .insert(network_team.team.id, ranking.clone());
+
+                    if let Err(err) = save_team_ranking(&self.team_ranking, true) {
+                        println!("Error while saving team ranking: {err}");
+                    }
+
+                    for player in network_team.players.iter() {
+                        let ranking = PlayerRanking::new(timestamp, player.clone());
+                        self.player_ranking.insert(player.id, ranking.clone());
+                    }
+
+                    if let Err(err) = save_player_ranking(&self.player_ranking, true) {
+                        println!("Error while saving player ranking: {err}");
+                    }
+
+                    self.top_team_ranking = self.get_top_team_ranking();
+                    self.top_player_ranking = self.get_top_player_ranking();
+
+                    // Check if there are new messages to send and append them to self.messages.
+                    self.messages.extend(load_relayer_messages()?);
+
+                    // Send messages starting from last sent message.
+                    let last_message_sent = self
+                        .last_message_sent_to_team
+                        .get(&network_team.team.id)
+                        .unwrap_or(&0);
+
+                    for (index, message) in self.messages.iter().enumerate() {
+                        if index < *last_message_sent {
+                            continue;
+                        }
+
+                        self.network_handler
+                            .send_relayer_message_to_team(message.clone(), network_team.team.id)?;
+                    }
+
+                    self.last_message_sent_to_team
+                        .insert(network_team.team.id, self.messages.len());
                 }
             }
             _ => {}
