@@ -3,9 +3,9 @@ use glam::UVec2;
 use crate::ui::UI_SCREEN_SIZE;
 
 pub(crate) const FRICTION_COEFF: f32 = 0.1;
-pub(crate) const THRUST_MOD: f32 = 1.25;
+pub(crate) const THRUST_MOD: f32 = 1.35;
 pub(crate) const FUEL_CONSUMPTION_MOD: f32 = 215_000.0;
-pub(crate) const MAX_SPACESHIP_SPEED_MOD: f32 = 0.125;
+pub(crate) const MAX_SPACESHIP_SPEED_MOD: f32 = 0.1;
 
 pub(crate) const ASTEROID_GENERATION_PROBABILITY: f64 = 0.05;
 pub(crate) const DIFFICULTY_FOR_ASTEROID_PLANET_GENERATION: usize = 90;
@@ -23,4 +23,4 @@ pub(crate) const BACKGROUND_IMAGE_SIZE: UVec2 = UVec2::new(240, 168);
 
 pub(crate) const MAX_LAYER: usize = 5;
 
-pub(crate) const MAX_ASTEROID_PLANET_IMAGE_TYPE: usize = 30;
+pub(crate) const MAX_ASTEROID_PLANET_IMAGE_NUMBER: usize = 30;
