@@ -1,9 +1,7 @@
 mod channel;
 mod client;
 mod server;
-mod ssh_event_handler;
 mod utils;
 
 pub use crate::ssh::channel::SSHWriterProxy;
 pub use crate::ssh::server::AppServer;
-pub use crate::ssh::ssh_event_handler::SSHEventHandler;
