@@ -26,20 +26,20 @@ const MAX_GIF_HEIGHT: u32 = 140;
 pub const FRAMES_PER_REVOLUTION: usize = 360;
 
 pub static SPINNING_BALL_GIF: Lazy<GifLines> = Lazy::new(|| {
+    const X_BLIT: u32 = MAX_GIF_WIDTH / 2 + 30;
+    const Y_BLIT: u32 = MAX_GIF_HEIGHT / 2 + 20;
     Gif::open("game/spinning_ball.gif".to_string())
         .expect("Left shot gif should open")
         .iter()
         .map(|img: &RgbaImage| {
             let base = &mut UNIVERSE_BACKGROUND.clone();
             // FIXME: Hardcoded from assets file, should be taken from Sol planet.
-            let x_blit = MAX_GIF_WIDTH / 2 + 30;
-            let y_blit = MAX_GIF_HEIGHT / 2 + 20;
+            base.copy_non_trasparent_from(img, X_BLIT, Y_BLIT)
+                .unwrap_or_else(|_| {
+                    panic!("Could not copy_non_trasparent_from at {X_BLIT} {Y_BLIT}")
+                });
 
-            // Blit img on base
-            base.copy_non_trasparent_from(&mut img.clone(), x_blit, y_blit)
-                .unwrap();
-
-            let center = (x_blit + img.width() / 2, y_blit + img.height() / 2);
+            let center = (X_BLIT + img.width() / 2, Y_BLIT + img.height() / 2);
             base.view(
                 center.0 - MAX_GIF_WIDTH / 2,
                 center.1 - MAX_GIF_HEIGHT / 2,
@@ -132,15 +132,17 @@ impl GifMap {
     }
 
     pub fn player_frame_lines(&mut self, player: &Player, tick: usize) -> AppResult<ImageLines> {
+        const TICKS_PER_FRAME: usize = 5;
+
         if let Some((version, lines)) = self.players_lines.get(&player.id) {
             if player.version == *version {
-                return Ok(lines[(tick / 8) % lines.len()].clone());
+                return Ok(lines[(tick / TICKS_PER_FRAME) % lines.len()].clone());
             }
         }
 
         let gif = self.player(player)?;
         let lines = gif.to_lines();
-        let frame = lines[(tick / 8) % lines.len()].clone();
+        let frame = lines[(tick / TICKS_PER_FRAME) % lines.len()].clone();
         self.players_lines
             .insert(player.id, (player.version, lines));
         Ok(frame)
@@ -163,7 +165,7 @@ impl GifMap {
                     let base = &mut UNIVERSE_BACKGROUND.clone();
 
                     // Blit img on base
-                    base.copy_non_trasparent_from(&img, x_blit, y_blit).unwrap();
+                    base.copy_non_trasparent_from(img, x_blit, y_blit).unwrap();
 
                     let center = (x_blit + img.width() / 2, y_blit + img.height() / 2);
                     base.view(
@@ -185,7 +187,7 @@ impl GifMap {
         tick: usize,
         world: &World,
     ) -> AppResult<ImageLines> {
-        if let Some(lines) = self.planets_zoom_in_lines.get(&planet_id) {
+        if let Some(lines) = self.planets_zoom_in_lines.get(planet_id) {
             return Ok(lines[tick % lines.len()].clone());
         }
 
@@ -298,6 +300,7 @@ impl GifMap {
             for idx in 0..planet.satellites.len() {
                 let satellite = world.get_planet_or_err(&planet.satellites[idx])?;
                 // Satellite img moves along an ellipse
+                // Can divide safely because if we enter the loop => planet.satellites.len() > 0.
                 let theta_0 =
                     idx as f32 * 2.0 * std::f32::consts::PI / planet.satellites.len() as f32;
 
@@ -345,7 +348,7 @@ impl GifMap {
         let gif = if planet.planet_type == PlanetType::Asteroid {
             Self::asteroid_zoom_out(&planet.filename)?
         } else {
-            Self::planet_zoom_out(&planet, world)?
+            Self::planet_zoom_out(planet, world)?
         };
         let lines = gif.to_lines();
         let frame = lines[tick % lines.len()].clone();
