@@ -10,23 +10,11 @@ use super::{
     utils::format_satoshi,
     widgets::*,
 };
-use crate::world::constants::MAX_PLAYERS_PER_GAME;
-use crate::world::planet::{AsteroidUpgrade, AsteroidUpgradeTarget};
 use crate::{
     game_engine::game::Game,
     store::load_game,
-    types::{AppResult, GameId, PlayerId, StorableResourceMap, SystemTimeTick},
-    world::{
-        position::{GamePosition, Position, MAX_POSITION},
-        skill::Rated,
-        spaceship::{SpaceshipComponent, SpaceshipUpgrade, SpaceshipUpgradeTarget},
-        types::{TeamBonus, TeamLocation},
-        world::World,
-    },
-};
-use crate::{
-    types::{PlanetId, TeamId},
-    world::{resources::Resource, role::CrewRole},
+    types::{AppResult, GameId, PlanetId, StorableResourceMap, SystemTimeTick, TeamId},
+    world::*,
 };
 use anyhow::anyhow;
 use core::fmt::Debug;
@@ -92,6 +80,7 @@ pub struct MyTeamPanel {
     current_planet_id: Option<PlanetId>,
     tick: usize,
     gif_map: GifMap,
+    // players_table: ClickableTable<'static>,
 }
 
 impl MyTeamPanel {
@@ -146,7 +135,15 @@ impl MyTeamPanel {
         .set_hover_text("View shipyard, improve your spaceship.");
 
         let mut view_asteroids_button = Button::new(
-            format!("Asteroids ({})", self.asteroid_ids.len()),
+            format!(
+                "Asteroids ({}{})",
+                self.asteroid_ids.len(),
+                if self.asteroid_ids.len() == MAX_NUM_ASTEROID_PER_TEAM {
+                    " MAX"
+                } else {
+                    ""
+                }
+            ),
             UiCallback::SetMyTeamPanelView {
                 view: MyTeamView::Asteroids,
             },
@@ -241,7 +238,10 @@ impl MyTeamPanel {
         frame.render_widget(
             Paragraph::new(vec![
                 Line::from(""),
-                Line::from(Span::styled(format!("Resource: Prices"), UiStyle::HEADER)),
+                Line::from(Span::styled(
+                    "Resource: Prices".to_string(),
+                    UiStyle::HEADER,
+                )),
                 Line::from(vec![
                     Span::styled("Fuel      ", Resource::FUEL.style()),
                     Span::styled(
@@ -391,7 +391,7 @@ impl MyTeamPanel {
 
         frame.render_widget(
             Paragraph::new(Span::styled(
-                format!("        Key        Buy               Sell        Prices"),
+                "        Key        Buy               Sell        Prices".to_string(),
                 UiStyle::HEADER,
             )),
             button_split[0],
@@ -437,13 +437,10 @@ impl MyTeamPanel {
             frame.render_widget(
                 Paragraph::new(Line::from(vec![
                     Span::styled(format!("{:<6} ", resource.to_string()), resource.style()),
+                    Span::styled(format!("{}", buy_ui_keys[button_split_idx]), UiStyle::OK),
+                    Span::raw("/".to_string()),
                     Span::styled(
-                        format!("{}", buy_ui_keys[button_split_idx].to_string()),
-                        UiStyle::OK,
-                    ),
-                    Span::raw(format!("/")),
-                    Span::styled(
-                        format!("{}", sell_ui_keys[button_split_idx].to_string()),
+                        format!("{}", sell_ui_keys[button_split_idx]),
                         UiStyle::ERROR,
                     ),
                 ])),
@@ -454,9 +451,9 @@ impl MyTeamPanel {
             );
             frame.render_widget(
                 Paragraph::new(Line::from(vec![
-                    Span::styled(format!("{}", buy_unit_cost), UiStyle::OK),
-                    Span::raw(format!("/")),
-                    Span::styled(format!("{}", sell_unit_cost), UiStyle::ERROR),
+                    Span::styled(format!("{buy_unit_cost}"), UiStyle::OK),
+                    Span::raw("/".to_string()),
+                    Span::styled(format!("{sell_unit_cost}"), UiStyle::ERROR),
                 ])),
                 resource_split[7].inner(Margin {
                     horizontal: 1,
@@ -467,7 +464,7 @@ impl MyTeamPanel {
             let max_buy_amount = own_team.max_resource_buy_amount(*resource, buy_unit_cost);
             for (idx, amount) in [1, 10, max_buy_amount as i32].iter().enumerate() {
                 if let Ok(btn) = trade_resource_button(
-                    &world,
+                    world,
                     *resource,
                     *amount,
                     buy_unit_cost,
@@ -485,7 +482,7 @@ impl MyTeamPanel {
             let max_sell_amount = own_team.max_resource_sell_amount(*resource);
             for (idx, amount) in [1, 10, max_sell_amount as i32].iter().enumerate() {
                 if let Ok(btn) = trade_resource_button(
-                    &world,
+                    world,
                     *resource,
                     -*amount,
                     sell_unit_cost,
@@ -724,10 +721,7 @@ impl MyTeamPanel {
         .set_hover_text("Change the training focus, which affects how player skills increase.")
         .set_hotkey(UiKey::TRAINING_FOCUS);
         if can_change_training_focus.is_err() {
-            training_button.disable(Some(format!(
-                "{}",
-                can_change_training_focus.unwrap_err().to_string()
-            )));
+            training_button.disable(Some(format!("{}", can_change_training_focus.unwrap_err())));
         }
         frame.render_interactive(training_button, top_button_split[1]);
 
@@ -742,7 +736,7 @@ impl MyTeamPanel {
             ),
             UiCallback::ToggleTeamAutonomousStrategyForLocalChallenges,
         )
-        .set_hover_text(format!("Accept automatically challenges from local teams.",))
+        .set_hover_text("Accept automatically challenges from local teams.".to_string())
         .set_hotkey(UiKey::TOGGLE_ACCEPT_LOCAL_CHALLENGES);
         frame.render_interactive(local_challenge_button, btm_split[1]);
 
@@ -757,7 +751,7 @@ impl MyTeamPanel {
             ),
             UiCallback::ToggleTeamAutonomousStrategyForNetworkChallenges,
         )
-        .set_hover_text(format!("Accept automatically challenges from local teams.",))
+        .set_hover_text("Accept automatically challenges from local teams.".to_string())
         .set_hotkey(UiKey::TOGGLE_ACCEPT_NETWORK_CHALLENGES);
         frame.render_interactive(network_challenge_button, btm_split[2]);
 
@@ -860,7 +854,7 @@ impl MyTeamPanel {
     ) -> AppResult<()> {
         frame.render_widget(default_block().title("Recent Games".to_string()), area);
 
-        if self.recent_games.len() == 0 {
+        if self.recent_games.is_empty() {
             return Ok(());
         }
         let own_team = world.get_own_team()?;
@@ -893,7 +887,11 @@ impl MyTeamPanel {
                     game.away_team_name,
                 );
 
-                let style = UiStyle::DEFAULT;
+                let style = if game.is_network {
+                    UiStyle::NETWORK
+                } else {
+                    UiStyle::DEFAULT
+                };
                 options.push((text, style));
             }
         }
@@ -918,9 +916,9 @@ impl MyTeamPanel {
                 current_game.timer.format(),
             ))
         } else {
-            if self.loaded_games.get(&game_id).is_none() {
+            if let std::collections::hash_map::Entry::Vacant(e) = self.loaded_games.entry(game_id) {
                 if let Ok(game) = load_game(game_id) {
-                    self.loaded_games.insert(game_id, game);
+                    e.insert(game);
                 } else {
                     log::error!("Failed to load game {game_id}");
                     return Ok(());
@@ -993,7 +991,7 @@ impl MyTeamPanel {
                         ""
                     }
                 )),
-                Line::from(format!("")),
+                Line::from(String::new()),
                 Line::from(Span::styled(game.home_team_name.clone(), UiStyle::HEADER)),
                 Line::from(format!(
                     "{:<18}{:<8}{:<8}{:<8}",
@@ -1248,13 +1246,11 @@ impl MyTeamPanel {
             },
             None => {
                 if can_be_upgraded {
-                    format!("{} -> {}", current, next)
+                    format!("{current} -> {next}")
+                } else if self.spaceship_upgrade_index == SpaceshipUpgradeTarget::MAX_INDEX - 1 {
+                    "Fully repaired".to_string()
                 } else {
-                    if self.spaceship_upgrade_index == SpaceshipUpgradeTarget::MAX_INDEX - 1 {
-                        "Fully repaired".to_string()
-                    } else {
-                        "Fully upgraded".to_string()
-                    }
+                    "Fully upgraded".to_string()
                 }
             }
         };
@@ -1280,7 +1276,7 @@ impl MyTeamPanel {
             let upgrade = SpaceshipUpgrade::new(target, bonus);
             for (resource, amount) in upgrade.cost().iter() {
                 let have = own_team.resources.value(resource);
-                let style = if amount.clone() > have {
+                let style = if *amount > have {
                     UiStyle::ERROR
                 } else {
                     UiStyle::OK
@@ -1288,7 +1284,7 @@ impl MyTeamPanel {
 
                 lines.push(Line::from(vec![
                     Span::styled(format!("  {:<7} ", resource.to_string()), resource.style()),
-                    Span::styled(format!("{}/{}", amount, have,), style),
+                    Span::styled(format!("{amount}/{have}"), style),
                 ]));
             }
         }
@@ -1318,7 +1314,7 @@ impl MyTeamPanel {
     ) -> AppResult<()> {
         frame.render_widget(default_block().title("Asteroids "), area);
 
-        if self.asteroid_ids.len() == 0 {
+        if self.asteroid_ids.is_empty() {
             frame.render_widget(
                 Paragraph::new("No asteroid has been found yet, keep exploring!")
                     .wrap(Wrap { trim: true }),
@@ -1371,47 +1367,105 @@ impl MyTeamPanel {
         if let Some(index) = self.asteroid_index {
             let asteroid_id = own_team.asteroid_ids[index];
             let asteroid = world.get_planet_or_err(&asteroid_id)?;
+            Self::render_available_asteroid_upgrades(asteroid, own_team, frame, world, split[1])?;
+        }
 
-            let upgrade_to_text = if asteroid
-                .upgrades
-                .contains(&AsteroidUpgradeTarget::TeleportationPad)
-            {
-                "Already built"
-            } else if asteroid.pending_upgrade.is_some() {
-                "Under construction"
-            } else {
-                ""
-            };
+        Ok(())
+    }
 
-            let mut lines = vec![
-                Line::from(""),
-                Line::from(Span::styled("Build Teleportation Pad", UiStyle::HEADER)).centered(),
-                Line::from(Span::raw(upgrade_to_text)).centered(),
-                Line::from(""),
-            ];
+    fn render_available_asteroid_upgrades(
+        asteroid: &Planet,
+        own_team: &Team,
+        frame: &mut UiFrame,
+        world: &World,
+        area: Rect,
+    ) -> AppResult<()> {
+        if asteroid
+            .upgrades
+            .contains(&AsteroidUpgradeTarget::TeleportationPad)
+        {
+            let split = Layout::vertical([3, 6]).split(area);
 
-            if !asteroid
-                .upgrades
-                .contains(&AsteroidUpgradeTarget::TeleportationPad)
-                && asteroid.pending_upgrade.is_none()
-            {
-                let bonus = TeamBonus::Upgrades.current_team_bonus(world, &own_team.id)?;
-                let upgrade = AsteroidUpgrade::new(AsteroidUpgradeTarget::TeleportationPad, bonus);
-                for (resource, amount) in upgrade.cost().iter() {
-                    let have = own_team.resources.value(resource);
-                    let style = if amount.clone() > have {
-                        UiStyle::ERROR
-                    } else {
-                        UiStyle::OK
-                    };
+            // Teleport button
+            let mut travel_to_planet_button = Button::new(
+                "Teleport",
+                UiCallback::TravelToPlanet {
+                    planet_id: asteroid.id,
+                },
+            )
+            .set_hotkey(UiKey::TRAVEL)
+            .set_hover_text(format!("Travel instantaneously to {}", asteroid.name));
 
-                    lines.push(Line::from(vec![
-                        Span::styled(format!("  {:<7} ", resource.to_string()), resource.style()),
-                        Span::styled(format!("{}/{}", amount, have,), style),
-                    ]));
-                }
+            if let Err(e) = own_team.can_teleport_to(asteroid) {
+                travel_to_planet_button.disable(Some(e.to_string()));
             }
 
+            let duration = world.travel_time_to_planet(own_team.id, asteroid.id)?;
+            if let Err(e) = own_team.can_travel_to_planet(asteroid, duration) {
+                travel_to_planet_button.disable(Some(e.to_string()));
+            }
+
+            frame.render_interactive(travel_to_planet_button, split[0]);
+        } else if matches!(
+            asteroid.pending_upgrade,
+            Some(AsteroidUpgrade {
+                target: AsteroidUpgradeTarget::TeleportationPad,
+                ..
+            })
+        ) {
+            let split = Layout::vertical([3, 6]).split(area);
+
+            // Pending upgrsade disabled button
+            let build_teleport_pod_button = Button::new("Building", UiCallback::None)
+                .set_hover_text(format!("Building teleportation pad on {}", asteroid.name))
+                .disabled(Some("Building teleportation pad"));
+
+            frame.render_interactive(build_teleport_pod_button, split[0]);
+        } else if !asteroid
+            .upgrades
+            .contains(&AsteroidUpgradeTarget::TeleportationPad)
+            && asteroid.pending_upgrade.is_none()
+        {
+            let split = Layout::vertical([3, 6]).split(area);
+
+            // Build button
+            let bonus = TeamBonus::Upgrades.current_team_bonus(world, &own_team.id)?;
+            let upgrade = AsteroidUpgrade::new(AsteroidUpgradeTarget::TeleportationPad, bonus);
+            let mut build_teleport_pod_button = Button::new(
+                "Build teleportation pod",
+                UiCallback::SetAsteroidUpgrade {
+                    asteroid_id: asteroid.id,
+                    upgrade,
+                },
+            )
+            .set_hotkey(UiKey::BUILD_TELEPORTATION_PAD)
+            .set_hover_text(format!(
+                "Build teleportation pad to travel instantaneously to {}",
+                asteroid.name
+            ));
+
+            let can_upgrade_asteroid = own_team.can_upgrade_asteroid(asteroid, &upgrade);
+            if let Err(e) = can_upgrade_asteroid.as_ref() {
+                build_teleport_pod_button.disable(Some(e.to_string()));
+            }
+
+            frame.render_interactive(build_teleport_pod_button, split[0]);
+
+            // Resources required
+            let mut lines = vec![];
+            for (resource, amount) in upgrade.cost().iter() {
+                let have = own_team.resources.value(resource);
+                let style = if *amount > have {
+                    UiStyle::ERROR
+                } else {
+                    UiStyle::OK
+                };
+
+                lines.push(Line::from(vec![
+                    Span::styled(format!("  {:<7} ", resource.to_string()), resource.style()),
+                    Span::styled(format!("{amount}/{have}"), style),
+                ]));
+            }
             frame.render_widget(Paragraph::new(lines), split[1]);
         }
 
@@ -1424,7 +1478,7 @@ impl MyTeamPanel {
         world: &World,
         area: Rect,
     ) -> AppResult<()> {
-        if self.asteroid_ids.len() == 0 {
+        if self.asteroid_ids.is_empty() {
             frame.render_widget(default_block(), area);
             return Ok(());
         }
@@ -1470,48 +1524,6 @@ impl MyTeamPanel {
             .planet_zoom_out_frame_lines(asteroid, 0, world)?;
         frame.render_widget(Paragraph::new(img_lines).centered(), split[0]);
 
-        let own_team = world.get_own_team()?;
-
-        if own_team.can_teleport_to(asteroid) {
-            let mut travel_to_planet_button = Button::new(
-                "Teleport",
-                UiCallback::TravelToPlanet {
-                    planet_id: asteroid_id,
-                },
-            )
-            .set_hotkey(UiKey::TRAVEL)
-            .set_hover_text(format!("Travel instantaneously to {}", asteroid.name));
-
-            let duration = world.travel_time_to_planet(own_team.id, asteroid_id)?;
-            if let Err(e) = own_team.can_travel_to_planet(&asteroid, duration) {
-                travel_to_planet_button.disable(Some(e.to_string()));
-            }
-
-            frame.render_interactive(travel_to_planet_button, split[1]);
-        } else {
-            let bonus = TeamBonus::Upgrades.current_team_bonus(world, &own_team.id)?;
-            let upgrade = AsteroidUpgrade::new(AsteroidUpgradeTarget::TeleportationPad, bonus);
-            let mut build_teleport_pod_button = Button::new(
-                "Build teleportation pod",
-                UiCallback::SetAsteroidUpgrade {
-                    asteroid_id,
-                    upgrade,
-                },
-            )
-            .set_hotkey(UiKey::BUILD_TELEPORTATION_POD)
-            .set_hover_text(format!(
-                "Build teleportation pad to travel instantaneously to {}",
-                asteroid.name
-            ));
-
-            let can_upgrade_asteroid = own_team.can_upgrade_asteroid(&asteroid, &upgrade);
-            if let Err(e) = can_upgrade_asteroid.as_ref() {
-                build_teleport_pod_button.disable(Some(e.to_string()));
-            }
-
-            frame.render_interactive(build_teleport_pod_button, split[1]);
-        }
-
         let b_split =
             Layout::horizontal([Constraint::Ratio(1, 2), Constraint::Ratio(1, 2)]).split(split[2]);
 
@@ -1530,20 +1542,20 @@ impl MyTeamPanel {
 
     fn render_player_buttons(
         &self,
-        players: Vec<PlayerId>,
+        players: &[&Player],
         frame: &mut UiFrame,
         world: &World,
         area: Rect,
     ) -> AppResult<()> {
         let own_team = world.get_own_team()?;
         let player_index = if let Some(index) = self.player_index {
-            index
+            index.min(players.len() - 1)
         } else {
             return Ok(());
         };
 
-        let player_id = players[player_index];
-        let player = world.get_player_or_err(&player_id)?;
+        let player = players[player_index];
+        let player_id = player.id;
         let button_splits = Layout::horizontal([
             Constraint::Length(12),
             Constraint::Length(12),
@@ -1558,7 +1570,7 @@ impl MyTeamPanel {
             horizontal: 1,
         }));
 
-        let can_set_crew_role = own_team.can_set_crew_role(&player);
+        let can_set_crew_role = own_team.can_set_crew_role(player);
 
         let mut captain_button = Button::new(
             "captain",
@@ -1577,7 +1589,7 @@ impl MyTeamPanel {
         .set_hotkey(UiKey::SET_CAPTAIN);
         if own_team.crew_roles.captain == Some(player.id) {
             captain_button = captain_button
-                .set_hover_text(format!("Remove player from captain role"))
+                .set_hover_text("Remove player from captain role".to_string())
                 .selected();
         } else if let Err(e) = can_set_crew_role.as_ref() {
             captain_button.disable(Some(e.to_string()));
@@ -1601,7 +1613,7 @@ impl MyTeamPanel {
         .set_hotkey(UiKey::SET_PILOT);
         if own_team.crew_roles.pilot == Some(player.id) {
             pilot_button = pilot_button
-                .set_hover_text(format!("Remove player from pilot role"))
+                .set_hover_text("Remove player from pilot role".to_string())
                 .selected();
         } else if let Err(e) = can_set_crew_role.as_ref() {
             pilot_button.disable(Some(e.to_string()));
@@ -1625,7 +1637,7 @@ impl MyTeamPanel {
         .set_hotkey(UiKey::SET_DOCTOR);
         if own_team.crew_roles.doctor == Some(player.id) {
             doctor_button = doctor_button
-                .set_hover_text(format!("Remove player from doctor role"))
+                .set_hover_text("Remove player from doctor role".to_string())
                 .selected();
         } else if let Err(e) = can_set_crew_role.as_ref() {
             doctor_button.disable(Some(e.to_string()));
@@ -1649,14 +1661,14 @@ impl MyTeamPanel {
         .set_hotkey(UiKey::SET_ENGINEER);
         if own_team.crew_roles.engineer == Some(player.id) {
             engineer_button = engineer_button
-                .set_hover_text(format!("Remove player from engineer role"))
+                .set_hover_text("Remove player from engineer role".to_string())
                 .selected();
         } else if let Err(e) = can_set_crew_role.as_ref() {
             engineer_button.disable(Some(e.to_string()));
         }
         frame.render_interactive(engineer_button, button_splits[3]);
 
-        let can_release = own_team.can_release_player(&player);
+        let can_release = own_team.can_release_player(player);
         let mut release_button = Button::new(
             format!("Fire {}", player.info.short_name()),
             UiCallback::PromptReleasePlayer { player_id },
@@ -1664,7 +1676,7 @@ impl MyTeamPanel {
         .set_hover_text("Fire pirate from the crew!")
         .set_hotkey(UiKey::FIRE);
         if can_release.is_err() {
-            release_button.disable(Some(format!("{}", can_release.unwrap_err().to_string())));
+            release_button.disable(Some(format!("{}", can_release.unwrap_err())));
         } else {
             release_button = release_button.block(default_block().border_style(UiStyle::WARNING));
         }
@@ -1678,15 +1690,14 @@ impl MyTeamPanel {
         Ok(())
     }
 
-    fn build_players_table(
-        &'_ self,
-        players: &Vec<PlayerId>,
-        world: &World,
+    fn build_players_table<'a>(
+        players: &'a Vec<&Player>,
+        world: &'a World,
         table_width: u16,
-    ) -> AppResult<ClickableTable<'_>> {
+    ) -> AppResult<ClickableTable<'a>> {
         let own_team = world.get_own_team()?;
         let header_cells = [
-            " Name",
+            "Name",
             "Overall",
             "Potential",
             "Current",
@@ -1700,12 +1711,13 @@ impl MyTeamPanel {
 
         // Calculate the available space for the players name in order to display the
         // full or shortened version.
-        let name_header_width = table_width - (9 + 10 + 10 + 10 + 9 + 15 + 17);
+        let name_header_width = table_width
+            .saturating_sub(9 + 10 + 10 + 10 + 9 + 15 + 17)
+            .max(1);
 
         let rows = players
             .iter()
-            .map(|id| {
-                let player = world.get_player(id).unwrap();
+            .map(|player| {
                 let skills = player.current_skill_array();
 
                 let current_role = match own_team.player_ids.iter().position(|id| *id == player.id)
@@ -1794,7 +1806,6 @@ impl MyTeamPanel {
                 } else {
                     player.info.short_name()
                 };
-
                 let cells = [
                     ClickableCell::from(name),
                     ClickableCell::from(overall),
@@ -1837,22 +1848,24 @@ impl MyTeamPanel {
         area: Rect,
     ) -> AppResult<()> {
         let own_team = world.get_own_team()?;
-        let mut players = own_team.player_ids.clone();
-        players.sort_by(|a, b| {
-            let a = world.get_player(a).unwrap();
-            let b = world.get_player(b).unwrap();
-            if a.rating() == b.rating() {
-                b.average_skill()
-                    .partial_cmp(&a.average_skill())
-                    .expect("Skill value should exist")
-            } else {
-                b.rating().cmp(&a.rating())
-            }
-        });
+        let sorted_players = own_team
+            .player_ids
+            .iter()
+            .map(|id| world.get_player(id).unwrap())
+            .collect_vec()
+            .sort_by_rating();
+
+        let player_index = if let Some(index) = self.player_index {
+            index.min(sorted_players.len() - 1)
+        } else {
+            return Ok(());
+        };
+        let player = sorted_players[player_index];
+
         let top_split =
             Layout::horizontal([Constraint::Min(10), Constraint::Length(60)]).split(area);
 
-        let table = self.build_players_table(&players, world, top_split[0].width)?;
+        let table = Self::build_players_table(&sorted_players, world, top_split[0].width)?;
         frame.render_stateful_interactive(
             table.block(default_block().title(format!(
                 "{} {} ↓/↑",
@@ -1863,16 +1876,6 @@ impl MyTeamPanel {
             &mut ClickableTableState::default().with_selected(self.player_index),
         );
 
-        if self.player_index.is_none() {
-            return Ok(());
-        }
-
-        let player_id = players[self.player_index.unwrap()];
-
-        let player = world
-            .get_player(&player_id)
-            .ok_or(anyhow!("Player {:?} not found", player_id))?;
-
         render_player_description(
             player,
             self.player_widget_view,
@@ -1931,7 +1934,7 @@ impl MyTeamPanel {
                 Constraint::Length(6),  //c
                 Constraint::Length(6),  //bench
                 Constraint::Length(6),  //bench
-                Constraint::Length(32), //auto-assign
+                Constraint::Length(30), //auto-assign
                 Constraint::Min(0),
             ])
             .split(table_bottom[1].inner(Margin {
@@ -1939,7 +1942,8 @@ impl MyTeamPanel {
                 horizontal: 1,
             }));
 
-            for idx in 0..MAX_PLAYERS_PER_GAME as usize {
+            let player_id = player.id;
+            for idx in 0..MAX_PLAYERS_PER_GAME {
                 let position = idx as Position;
                 let rect = position_button_splits[idx];
                 let mut button = Button::new(
@@ -1977,7 +1981,7 @@ impl MyTeamPanel {
                     .set_hover_text("Auto-assign players' initial position.")
                     .set_hotkey(UiKey::AUTO_ASSIGN);
             frame.render_interactive(auto_assign_button, position_button_splits[7]);
-            self.render_player_buttons(players, frame, world, table_bottom[2])?;
+            self.render_player_buttons(&sorted_players, frame, world, table_bottom[2])?;
         }
         Ok(())
     }
@@ -1998,7 +2002,7 @@ impl MyTeamPanel {
         );
 
         render_spaceship_description(
-            &own_team,
+            own_team,
             world,
             world.team_rating(&own_team.id).unwrap_or_default(),
             true,
@@ -2033,8 +2037,8 @@ impl MyTeamPanel {
             .saturating_sub(world.last_tick_short_interval)
             .formatted();
         render_spaceship_upgrade(
-            &own_team,
-            &upgrade,
+            own_team,
+            upgrade,
             true,
             &mut self.gif_map,
             self.tick,
@@ -2087,7 +2091,7 @@ impl MyTeamPanel {
             let bonus = TeamBonus::Upgrades.current_team_bonus(world, &own_team.id)?;
             let upgrade = SpaceshipUpgrade::new(target, bonus);
             render_spaceship_upgrade(
-                &own_team,
+                own_team,
                 &upgrade,
                 false,
                 &mut self.gif_map,
@@ -2108,7 +2112,7 @@ impl MyTeamPanel {
             }
         } else {
             render_spaceship_description(
-                &own_team,
+                own_team,
                 world,
                 world.team_rating(&own_team.id).unwrap_or_default(),
                 true,
@@ -2219,15 +2223,15 @@ impl Screen for MyTeamPanel {
             _ => None,
         };
 
-        if self.planet_markets.len() == 0 || world.dirty_ui {
+        if self.planet_markets.is_empty() || world.dirty_ui {
             self.planet_markets = world
                 .planets
                 .iter()
                 .filter(|(_, planet)| planet.total_population() > 0)
                 .sorted_by(|(_, a), (_, b)| a.name.cmp(&b.name))
-                .map(|(id, _)| id.clone())
+                .map(|(id, _)| *id)
                 .collect::<Vec<PlanetId>>();
-            if self.planet_index.is_none() && self.planet_markets.len() > 0 {
+            if self.planet_index.is_none() && !self.planet_markets.is_empty() {
                 self.planet_index = Some(0);
             }
         }
@@ -2236,7 +2240,7 @@ impl Screen for MyTeamPanel {
             self.asteroid_ids = own_team.asteroid_ids.clone();
         }
 
-        self.asteroid_index = if self.asteroid_ids.len() > 0 {
+        self.asteroid_index = if !self.asteroid_ids.is_empty() {
             if let Some(index) = self.asteroid_index {
                 Some(index % self.asteroid_ids.len())
             } else {
@@ -2246,7 +2250,7 @@ impl Screen for MyTeamPanel {
             None
         };
 
-        self.player_index = if own_team.player_ids.len() > 0 {
+        self.player_index = if !own_team.player_ids.is_empty() {
             if let Some(index) = self.player_index {
                 Some(index % own_team.player_ids.len())
             } else {
@@ -2281,7 +2285,6 @@ impl Screen for MyTeamPanel {
             self.challenge_teams = world
                 .teams
                 .keys()
-                .into_iter()
                 .filter(|&id| {
                     let team = world.get_team_or_err(id).unwrap();
                     own_team.can_challenge_local_team(team).is_ok()
@@ -2298,9 +2301,11 @@ impl Screen for MyTeamPanel {
                     .partial_cmp(&world.team_rating(&a.id).unwrap_or_default())
                     .unwrap()
             });
+
+            // self.players_table = Self::build_players_table(players, world, table_width)
         }
 
-        self.game_index = if self.recent_games.len() > 0 {
+        self.game_index = if !self.recent_games.is_empty() {
             if let Some(index) = self.game_index {
                 Some(index % self.recent_games.len())
             } else {
@@ -2352,9 +2357,7 @@ impl Screen for MyTeamPanel {
         key_event: crossterm::event::KeyEvent,
         _world: &World,
     ) -> Option<UiCallback> {
-        if self.planet_index.is_none() {
-            return None;
-        }
+        self.planet_index?;
 
         match key_event.code {
             KeyCode::Up => {
@@ -2424,18 +2427,16 @@ impl SplitPanel for MyTeamPanel {
             } else {
                 self.player_index = None;
             }
+        } else if self.active_list == PanelList::Bottom && self.view == MyTeamView::Games {
+            self.game_index = Some(index % self.max_index());
+        } else if self.active_list == PanelList::Bottom && self.view == MyTeamView::Market {
+            self.planet_index = Some(index % self.max_index());
+        } else if self.active_list == PanelList::Bottom && self.view == MyTeamView::Shipyard {
+            self.spaceship_upgrade_index = index % self.max_index();
+        } else if self.active_list == PanelList::Bottom && self.view == MyTeamView::Asteroids {
+            self.asteroid_index = Some(index % self.max_index());
         } else {
-            if self.active_list == PanelList::Bottom && self.view == MyTeamView::Games {
-                self.game_index = Some(index % self.max_index());
-            } else if self.active_list == PanelList::Bottom && self.view == MyTeamView::Market {
-                self.planet_index = Some(index % self.max_index());
-            } else if self.active_list == PanelList::Bottom && self.view == MyTeamView::Shipyard {
-                self.spaceship_upgrade_index = index % self.max_index();
-            } else if self.active_list == PanelList::Bottom && self.view == MyTeamView::Asteroids {
-                self.asteroid_index = Some(index % self.max_index());
-            } else {
-                self.player_index = Some(index % self.max_index());
-            }
+            self.player_index = Some(index % self.max_index());
         }
     }
 }
