@@ -39,7 +39,7 @@ pub struct SplashScreen {
     gif_map: GifMap,
 }
 
-const QUOTES: [&'static str;19] = [
+const QUOTES: [&str;19] = [
     " “What cannot be destroyed can, nonetheless, be diverted, frozen, transformed, and gradually deprived of its substance - which in the case of states, is ultimately their capacity to inspire terror.” - D. Graeber",
     " “Aber der Staat lügt in allen Zungen des Guten und Bösen; und was er auch redet, er lügt—und was er auch hat, gestohlen hat er's.” - F. Nietzsche",
     " “That is what I have always understood to be the essence of anarchism: the conviction that the burden of proof has to be placed on authority, and that it should be dismantled if that burden cannot be met.” - N. Chomsky",
@@ -62,7 +62,7 @@ const QUOTES: [&'static str;19] = [
     " “Chi trova il coraggio di costruire la propria esistenza nel mare mosso dell’incerto riuscirà più facilmente a trovare il proprio spazio nel presente di chi invece tenta di gettare l’ancora verso i lidi di epoche passate.” - Alexander Langer"
     ];
 
-const TITLE: [&'static str; 13] = [
+const TITLE: [&str; 13] = [
     "            ██████╗ ███████╗██████╗ ███████╗██╗     ███████╗           ",
     "            ██╔══██╗██╔════╝██╔══██╗██╔════╝██║     ██╔════╝           ",
     "            ██████╔╝█████╗  ██████╔╝█████╗  ██║     ███████╗           ",
@@ -123,6 +123,7 @@ impl SplashScreen {
         match index {
             0 => UiCallback::ContinueGame,
             1 => UiCallback::NewGame,
+            #[cfg(feature = "audio")]
             2 => UiCallback::ToggleAudio,
             _ => UiCallback::QuitGame,
         }
@@ -203,10 +204,10 @@ impl Screen for SplashScreen {
         .split(split[3]);
 
         let mut gif_lines = if self.index == 0 {
-            SPINNING_BALL_GIF[(self.tick / 4) % SPINNING_BALL_GIF.len()].clone()
+            SPINNING_BALL_GIF[(self.tick) % SPINNING_BALL_GIF.len()].clone()
         } else {
             self.gif_map
-                .planet_zoom_in_frame_lines(&SOL_ID, self.tick / 3, world)?
+                .planet_zoom_in_frame_lines(&SOL_ID, self.tick, world)?
         };
 
         let offset = if gif_lines.len() > split[3].height as usize {
@@ -309,6 +310,7 @@ impl Screen for SplashScreen {
                     return Some(UiCallback::NewGame);
                 }
                 //options
+                #[cfg(feature = "audio")]
                 2 => {
                     return Some(UiCallback::ToggleAudio);
                 }
