@@ -4,27 +4,32 @@ use super::galaxy_panel::GalaxyPanel;
 use super::popup_message::PopupMessage;
 use super::space_screen::SpaceScreen;
 use super::splash_screen::SplashScreen;
+
+use super::swarm_panel::{SwarmPanel, SwarmPanelEvent};
 use super::traits::SplitPanel;
 use super::ui_callback::{CallbackRegistry, UiCallback};
 use super::ui_frame::UiFrame;
-use super::utils::SwarmPanelEvent;
 use super::widgets::default_block;
 use super::{
     game_panel::GamePanel, my_team_panel::MyTeamPanel, new_team_screen::NewTeamScreen,
-    player_panel::PlayerListPanel, swarm_panel::SwarmPanel, team_panel::TeamListPanel,
-    traits::Screen,
+    player_panel::PlayerListPanel, team_panel::TeamListPanel, traits::Screen,
 };
-use crate::audio::music_player::MusicPlayer;
-use crate::audio::AudioPlayerState;
-use crate::types::{AppResult, SystemTimeTick, Tick};
+#[cfg(feature = "audio")]
+use crate::audio::{music_player::MusicPlayer, AudioPlayerState};
+
+use crate::types::Tick;
+use crate::types::{AppResult, SystemTimeTick};
 use crate::world::world::World;
 use core::fmt::Debug;
 use itertools::Itertools;
+
 use libp2p::PeerId;
 use ratatui::layout::Rect;
 use ratatui::style::{Color, Style};
 use ratatui::text::{Line, Span};
-use ratatui::widgets::{Clear, Paragraph};
+use ratatui::widgets::Clear;
+#[cfg(feature = "audio")]
+use ratatui::widgets::Paragraph;
 use ratatui::{
     layout::{Constraint, Layout},
     Frame,
@@ -52,6 +57,7 @@ pub enum UiTab {
     Pirates,
     Galaxy,
     Games,
+
     Swarm,
 }
 
@@ -61,13 +67,14 @@ pub struct Ui {
     ui_tabs: Vec<UiTab>,
     pub tab_index: usize,
     debug_view: bool,
-    last_update: Instant,
+    last_render: Instant,
     pub splash_screen: SplashScreen,
     pub new_team_screen: NewTeamScreen,
     pub space_screen: SpaceScreen,
     pub player_panel: PlayerListPanel,
     pub team_panel: TeamListPanel,
     pub game_panel: GamePanel,
+
     pub swarm_panel: SwarmPanel,
     pub my_team_panel: MyTeamPanel,
     pub galaxy_panel: GalaxyPanel,
@@ -82,18 +89,19 @@ impl Ui {
         let player_panel = PlayerListPanel::new();
         let team_panel = TeamListPanel::new();
         let game_panel = GamePanel::new();
+
         let swarm_panel = SwarmPanel::new();
         let my_team_panel = MyTeamPanel::new();
         let new_team_screen = NewTeamScreen::new();
         let galaxy_panel = GalaxyPanel::new();
 
-        let mut ui_tabs = vec![];
-
-        ui_tabs.push(UiTab::MyTeam);
-        ui_tabs.push(UiTab::Crews);
-        ui_tabs.push(UiTab::Pirates);
-        ui_tabs.push(UiTab::Galaxy);
-        ui_tabs.push(UiTab::Games);
+        let mut ui_tabs = vec![
+            UiTab::MyTeam,
+            UiTab::Crews,
+            UiTab::Pirates,
+            UiTab::Galaxy,
+            UiTab::Games,
+        ];
 
         if !disable_network {
             ui_tabs.push(UiTab::Swarm);
@@ -106,13 +114,14 @@ impl Ui {
             ui_tabs,
             tab_index: 0,
             debug_view: false,
-            last_update: Instant::now(),
+            last_render: Instant::now(),
             splash_screen,
             new_team_screen,
             space_screen,
             player_panel,
             team_panel,
             game_panel,
+
             swarm_panel,
             my_team_panel,
             galaxy_panel,
@@ -122,6 +131,15 @@ impl Ui {
         }
     }
 
+    pub fn push_chat_event(&mut self, timestamp: Tick, peer_id: Option<PeerId>, text: String) {
+        let event = SwarmPanelEvent {
+            timestamp,
+            peer_id,
+            text,
+        };
+        self.swarm_panel.push_chat_event(event);
+    }
+
     pub fn push_popup(&mut self, popup_message: PopupMessage) {
         // Avoid pushing twice the same popup
         if let Some(last_popup) = self.popup_messages.last().as_ref() {
@@ -154,11 +172,9 @@ impl Ui {
             }
         }
 
-        // Insert new popup at the front so it becomes the active one
-        self.popup_messages.insert(0, popup_message);
+        self.popup_messages.push(popup_message);
         if self.popup_messages.len() >= MAX_POPUP_MESSAGES {
-            // Remove a skippable message starting from the end (oldest) so we keep the newest
-            for index in (0..self.popup_messages.len()).rev() {
+            for index in 0..self.popup_messages.len() {
                 if self.popup_messages[index].is_skippable() {
                     self.popup_messages.remove(index);
                     break;
@@ -168,7 +184,6 @@ impl Ui {
     }
 
     pub fn push_popup_to_top(&mut self, popup_message: PopupMessage) {
-        // Ensure popup is placed at the top (index 0)
         self.popup_messages.insert(0, popup_message);
     }
 
@@ -211,6 +226,7 @@ impl Ui {
                 UiTab::Pirates => &self.player_panel,
                 UiTab::Galaxy => &self.galaxy_panel,
                 UiTab::Games => &self.game_panel,
+
                 UiTab::Swarm => &self.swarm_panel,
             },
             UiState::SpaceAdventure => &self.space_screen,
@@ -227,6 +243,7 @@ impl Ui {
                 UiTab::Pirates => Some(&mut self.player_panel),
                 UiTab::Galaxy => Some(&mut self.galaxy_panel),
                 UiTab::Games => Some(&mut self.game_panel),
+
                 UiTab::Swarm => Some(&mut self.swarm_panel),
             },
         }
@@ -242,6 +259,7 @@ impl Ui {
                 UiTab::Pirates => &mut self.player_panel,
                 UiTab::Galaxy => &mut self.galaxy_panel,
                 UiTab::Games => &mut self.game_panel,
+
                 UiTab::Swarm => &mut self.swarm_panel,
             },
             UiState::SpaceAdventure => &mut self.space_screen,
@@ -254,39 +272,30 @@ impl Ui {
         world: &World,
     ) -> Option<UiCallback> {
         match key_event.code {
-            UiKey::ESC => {
-                return Some(UiCallback::PromptQuit);
-            }
+            UiKey::ESC => Some(UiCallback::PromptQuit),
 
-            UiKey::UI_DEBUG_MODE => {
-                return Some(UiCallback::ToggleUiDebugMode);
-            }
+            UiKey::UI_DEBUG_MODE => Some(UiCallback::ToggleUiDebugMode),
 
-            UiKey::NEXT_TAB if self.state == UiState::Main && self.popup_messages.len() == 0 => {
+            UiKey::NEXT_TAB if self.state == UiState::Main && self.popup_messages.is_empty() => {
                 self.next_tab();
                 None
             }
 
             UiKey::PREVIOUS_TAB
-                if self.state == UiState::Main && self.popup_messages.len() == 0 =>
+                if self.state == UiState::Main && self.popup_messages.is_empty() =>
             {
                 self.previous_tab();
                 None
             }
             _ => {
                 // Special handling for space screen. It takes precedence over popups.
-                match self.state {
-                    UiState::SpaceAdventure => {
-                        if let Some(callback) =
-                            self.space_screen.handle_key_events(key_event, world)
-                        {
-                            return Some(callback);
-                        }
+                if self.state == UiState::SpaceAdventure {
+                    if let Some(callback) = self.space_screen.handle_key_events(key_event, world) {
+                        return Some(callback);
                     }
-                    _ => {}
                 }
 
-                if self.popup_messages.len() > 0 {
+                if !self.popup_messages.is_empty() {
                     return self.popup_messages[0].consumes_input(&mut self.popup_input, key_event);
                 }
                 self.popup_input.move_cursor(CursorMove::End);
@@ -321,21 +330,28 @@ impl Ui {
         self.tab_index = (self.tab_index + self.ui_tabs.len() - 1) % self.ui_tabs.len();
     }
 
-    pub fn update(&mut self, world: &World, audio_player: Option<&MusicPlayer>) -> AppResult<()> {
+    pub fn update(
+        &mut self,
+        world: &World,
+        #[cfg(feature = "audio")] audio_player: Option<&MusicPlayer>,
+    ) -> AppResult<()> {
         self.inner_registry.clear();
         match self.state {
             UiState::Splash => {
-                // This is only to get a nice view in the splash screen
-                let audio_state = if let Some(player) = audio_player {
-                    if player.is_playing() {
-                        AudioPlayerState::Playing
+                #[cfg(feature = "audio")]
+                {
+                    // This is only to get a nice view in the splash screen
+                    let audio_state = if let Some(player) = audio_player {
+                        if player.is_playing() {
+                            AudioPlayerState::Playing
+                        } else {
+                            AudioPlayerState::Paused
+                        }
                     } else {
-                        AudioPlayerState::Paused
-                    }
-                } else {
-                    AudioPlayerState::Disabled
-                };
-                self.splash_screen.set_audio_player_state(audio_state);
+                        AudioPlayerState::Disabled
+                    };
+                    self.splash_screen.set_audio_player_state(audio_state);
+                }
                 self.splash_screen.update(world)?
             }
             UiState::NewTeam => self.new_team_screen.update(world)?,
@@ -347,6 +363,7 @@ impl Ui {
                 self.player_panel.update(world)?;
                 self.game_panel.update(world)?;
                 self.galaxy_panel.update(world)?;
+
                 self.swarm_panel.update(world)?;
             }
             UiState::SpaceAdventure => self.space_screen.update(world)?,
@@ -356,10 +373,15 @@ impl Ui {
     }
 
     /// Renders the user interface widgets.
-    pub fn render(&mut self, frame: &mut Frame, world: &World, audio_player: Option<&MusicPlayer>) {
+    pub fn render(
+        &mut self,
+        frame: &mut Frame,
+        world: &World,
+        #[cfg(feature = "audio")] audio_player: Option<&MusicPlayer>,
+    ) {
         let mut ui_frame = UiFrame::new(frame);
         ui_frame.set_hovering(self.inner_registry.hovering());
-        if self.popup_messages.len() > 0 {
+        if !self.popup_messages.is_empty() {
             ui_frame.set_max_layer(1);
         } else {
             ui_frame.set_max_layer(0);
@@ -446,32 +468,30 @@ impl Ui {
         };
 
         if let Err(err) = render_result {
-            self.push_log_event(
-                Tick::now(),
-                None,
-                format!("Render error\n{}", err.to_string()),
-            );
+            self.push_log_event(Tick::now(), None, format!("Render error\n{err}"));
         }
 
         // Render footer
-        self.render_footer(&mut ui_frame, world, audio_player, split[1]);
+        self.render_footer(
+            &mut ui_frame,
+            world,
+            #[cfg(feature = "audio")]
+            audio_player,
+            split[1],
+        );
 
         if let Err(err) = self.render_popup_messages(&mut ui_frame, screen_area) {
-            self.push_log_event(
-                Tick::now(),
-                None,
-                format!("Popup render error\n{}", err.to_string()),
-            );
-            log::error!("Popup render error\n{}", err.to_string());
+            self.push_log_event(Tick::now(), None, format!("Popup render error\n{err}"));
+            log::error!("Popup render error\n{err}");
         }
-        self.last_update = Instant::now();
+        self.last_render = Instant::now();
 
         self.inner_registry = ui_frame.callback_registry().clone();
     }
 
     fn render_popup_messages(&mut self, frame: &mut UiFrame, area: Rect) -> AppResult<()> {
         // Render popup message
-        if self.popup_messages.len() > 0 {
+        if !self.popup_messages.is_empty() {
             self.popup_messages[0].render(frame, area, &mut self.popup_input)?;
         }
         Ok(())
@@ -481,7 +501,7 @@ impl Ui {
         &self,
         frame: &mut UiFrame,
         world: &World,
-        audio_player: Option<&MusicPlayer>,
+        #[cfg(feature = "audio")] audio_player: Option<&MusicPlayer>,
         area: Rect,
     ) {
         frame.render_widget(Clear, area);
@@ -506,7 +526,7 @@ impl Ui {
         }
 
         let extra_spans = if self.debug_view {
-            let fps = (1.0 / self.last_update.elapsed().as_secs_f64()).round() as u32;
+            let fps = (1.0 / self.last_render.elapsed().as_secs_f64()).round() as u32;
             let world_size = world.serialized_size / 1024;
 
             let mut spans = vec![
@@ -549,12 +569,13 @@ impl Ui {
             split[0],
         );
 
+        #[cfg(feature = "audio")]
         if let Some(audio_player) = &audio_player {
             frame.render_interactive(
                 Button::no_box(
                     format!(
                         " {}: Turn radio {} ",
-                        UiKey::TOGGLE_AUDIO.to_string(),
+                        UiKey::TOGGLE_AUDIO,
                         if audio_player.is_playing() {
                             "off"
                         } else {
@@ -569,7 +590,7 @@ impl Ui {
 
             frame.render_interactive(
                 Button::no_box(
-                    format!(" {} ", UiKey::PREVIOUS_RADIO.to_string()),
+                    format!(" {} ", UiKey::PREVIOUS_RADIO),
                     UiCallback::PreviousRadio,
                 )
                 .set_hotkey(UiKey::PREVIOUS_RADIO),
@@ -577,11 +598,8 @@ impl Ui {
             );
 
             frame.render_interactive(
-                Button::no_box(
-                    format!(" {} ", UiKey::NEXT_RADIO.to_string()),
-                    UiCallback::NextRadio,
-                )
-                .set_hotkey(UiKey::NEXT_RADIO),
+                Button::no_box(format!(" {} ", UiKey::NEXT_RADIO), UiCallback::NextRadio)
+                    .set_hotkey(UiKey::NEXT_RADIO),
                 split[3],
             );
             if audio_player.is_playing() {
