@@ -1,19 +1,20 @@
+use super::swarm_panel::SwarmView;
 use super::{
     galaxy_panel::ZoomLevel,
     my_team_panel::MyTeamView,
     new_team_screen::CreationState,
     player_panel::PlayerView,
     popup_message::PopupMessage,
-    swarm_panel::SwarmView,
     team_panel::TeamView,
     traits::{Screen, SplitPanel},
     ui::{UiState, UiTab},
 };
+
+use crate::network::{challenge::Challenge, trade::Trade};
 use crate::{
     app::App,
     game_engine::{tactic::Tactic, types::TeamInGame},
     image::color_map::{ColorMap, ColorPreset},
-    network::{challenge::Challenge, trade::Trade},
     space_adventure::{ControllableSpaceship, PlayerInput, SpaceAdventure},
     types::{
         AppCallback, AppResult, GameId, PlanetId, PlayerId, ResourceMap, StorableResourceMap,
@@ -22,7 +23,7 @@ use crate::{
     world::{
         constants::*,
         jersey::{Jersey, JerseyStyle},
-        planet::{AsteroidUpgrade, AsteroidUpgradeTarget},
+        planet::AsteroidUpgrade,
         player::Trait,
         resources::Resource,
         role::CrewRole,
@@ -90,9 +91,11 @@ pub enum UiCallback {
     ChallengeTeam {
         team_id: TeamId,
     },
+
     AcceptChallenge {
         challenge: Challenge,
     },
+
     DeclineChallenge {
         challenge: Challenge,
     },
@@ -100,12 +103,15 @@ pub enum UiCallback {
         proposer_player_id: PlayerId,
         target_player_id: PlayerId,
     },
+
     AcceptTrade {
         trade: Trade,
     },
+
     DeclineTrade {
         trade: Trade,
     },
+
     GoToTrade {
         trade: Trade,
     },
@@ -128,9 +134,13 @@ pub enum UiCallback {
     NewGame,
     ContinueGame,
     QuitGame,
+    #[cfg(feature = "audio")]
     ToggleAudio,
+    #[cfg(feature = "audio")]
     PreviousRadio,
+    #[cfg(feature = "audio")]
     NextRadio,
+
     SetSwarmPanelView {
         topic: SwarmView,
     },
@@ -312,7 +322,7 @@ impl UiCallback {
                 .world
                 .get_player_or_err(&player_id)?
                 .team
-                .ok_or(anyhow!("Player {:?} has no team", player_id))?;
+                .ok_or(anyhow!("Player {player_id:?} has no team"))?;
 
             app.ui.team_panel.reset_view();
 
@@ -467,11 +477,12 @@ impl UiCallback {
             // Challenge to network team.
             if let Some(peer_id) = team.peer_id {
                 own_team.can_challenge_network_team(team)?;
-                let challenge = app
-                    .network_handler
-                    .as_mut()
-                    .ok_or(anyhow!("Network handler is not initialized"))?
-                    .send_new_challenge(&app.world, peer_id, team.id)?;
+                let challenge = app.network_handler.send_new_challenge(
+                    &app.world,
+                    peer_id,
+                    team.id,
+                    app.app_version(),
+                )?;
 
                 let own_team = app.world.get_own_team_mut()?;
                 own_team.add_sent_challenge(challenge);
@@ -492,16 +503,16 @@ impl UiCallback {
             {
                 0 => (
                     TeamInGame::from_team_id(&own_team_id, &app.world.teams, &app.world.players)
-                        .ok_or(anyhow!("Own team {:?} not found", own_team_id))?,
+                        .ok_or(anyhow!("Own team {own_team_id:?} not found"))?,
                     TeamInGame::from_team_id(&team_id, &app.world.teams, &app.world.players)
-                        .ok_or(anyhow!("Team {:?} not found", team_id))?,
+                        .ok_or(anyhow!("Team {team_id:?} not found"))?,
                 ),
 
                 1 => (
                     TeamInGame::from_team_id(&team_id, &app.world.teams, &app.world.players)
-                        .ok_or(anyhow!("Team {:?} not found", team_id))?,
+                        .ok_or(anyhow!("Team {team_id:?} not found"))?,
                     TeamInGame::from_team_id(&own_team_id, &app.world.teams, &app.world.players)
-                        .ok_or(anyhow!("Own team {:?} not found", own_team_id))?,
+                        .ok_or(anyhow!("Own team {own_team_id:?} not found"))?,
                 ),
                 _ => unreachable!(),
             };
@@ -513,7 +524,7 @@ impl UiCallback {
             app.ui.game_panel.update(&app.world)?;
             app.ui.game_panel.set_active_game(game_id)?;
             app.ui.switch_to(super::ui::UiTab::Games);
-            return Ok(Some("Challenge accepted".to_string()));
+            Ok(Some("Challenge accepted".to_string()))
         })
     }
 
@@ -533,11 +544,12 @@ impl UiCallback {
 
             // Network trade
             if let Some(peer_id) = target_team.peer_id {
-                let trade = app
-                    .network_handler
-                    .as_mut()
-                    .ok_or(anyhow!("Network handler is not initialized"))?
-                    .send_new_trade(&app.world, peer_id, proposer_player_id, target_player_id)?;
+                let trade = app.network_handler.send_new_trade(
+                    &app.world,
+                    peer_id,
+                    proposer_player_id,
+                    target_player_id,
+                )?;
                 let own_team = app.world.get_own_team_mut()?;
                 own_team.add_sent_trade(trade);
                 return Ok(Some("Trade offer sent".to_string()));
@@ -557,7 +569,7 @@ impl UiCallback {
 
                 return Ok(Some("Trade accepted".to_string()));
             }
-            return Ok(Some("Trade Rejected".to_string()));
+            Ok(Some("Trade Rejected".to_string()))
         })
     }
     fn next_ui_tab() -> AppCallback {
@@ -708,7 +720,7 @@ impl UiCallback {
             let duration = app
                 .world
                 .travel_time_to_planet(own_team.id, target_planet.id)?;
-            own_team.can_travel_to_planet(&target_planet, duration)?;
+            own_team.can_travel_to_planet(target_planet, duration)?;
             let distance = app
                 .world
                 .distance_between_planets(current_planet.id, target_planet.id)?;
@@ -720,11 +732,7 @@ impl UiCallback {
                 distance,
             };
 
-            let is_teleporting = if duration <= TELEPORT_MAX_DURATION {
-                true
-            } else {
-                false
-            };
+            let is_teleporting = duration <= TELEPORT_MAX_DURATION;
 
             if is_teleporting {
                 let rum_consumed = own_team.player_ids.len() as u32;
@@ -753,7 +761,7 @@ impl UiCallback {
 
             let pirate_jersey = Jersey {
                 style: JerseyStyle::Pirate,
-                color: own_team.jersey.color.clone(),
+                color: own_team.jersey.color,
             };
 
             for player in own_team.player_ids.iter() {
@@ -806,7 +814,7 @@ impl UiCallback {
 
             let pirate_jersey = Jersey {
                 style: JerseyStyle::Pirate,
-                color: own_team.jersey.color.clone(),
+                color: own_team.jersey.color,
             };
 
             for player_id in own_team.player_ids.iter() {
@@ -840,10 +848,7 @@ impl UiCallback {
 
     fn send(message: String) -> AppCallback {
         Box::new(move |app: &mut App| {
-            app.network_handler
-                .as_mut()
-                .ok_or(anyhow!("Network handler is not initialized"))?
-                .send_message(message.clone())?;
+            app.network_handler.send_message(message.clone())?;
 
             Ok(None)
         })
@@ -980,12 +985,11 @@ impl UiCallback {
             asteroid.version += 1;
             asteroid.upgrades.push(upgrade.target);
 
-            let message = match upgrade.target {
-                AsteroidUpgradeTarget::TeleportationPad => format!(
-                    "Teleportation pad construction on {} completed!",
-                    asteroid.name
-                ),
-            };
+            let message = format!(
+                "{} construction on {} completed!",
+                upgrade.target.name(),
+                asteroid.name
+            );
             app.world.planets.insert(asteroid.id, asteroid);
 
             app.ui.push_popup(PopupMessage::Ok {
@@ -1066,7 +1070,7 @@ impl UiCallback {
             UiCallback::SetTeamTactic { tactic } => {
                 let own_team = app.world.get_own_team()?;
                 let mut team = own_team.clone();
-                team.game_tactic = tactic.clone();
+                team.game_tactic = *tactic;
                 app.world.teams.insert(team.id, team);
                 app.world.dirty = true;
                 app.world.dirty_ui = true;
@@ -1099,8 +1103,6 @@ impl UiCallback {
             UiCallback::AcceptChallenge { challenge } => {
                 if let Err(e) = app
                     .network_handler
-                    .as_mut()
-                    .ok_or(anyhow!("Network handler is not initialized"))?
                     .accept_challenge(&app.world, challenge.clone())
                 {
                     let own_team = app.world.get_own_team_mut()?;
@@ -1120,10 +1122,7 @@ impl UiCallback {
                 Ok(None)
             }
             UiCallback::DeclineChallenge { challenge } => {
-                app.network_handler
-                    .as_mut()
-                    .ok_or(anyhow!("Network handler is not initialized"))?
-                    .decline_challenge(challenge.clone())?;
+                app.network_handler.decline_challenge(challenge.clone())?;
                 let own_team = app.world.get_own_team_mut()?;
                 own_team.remove_challenge(
                     challenge.home_team_in_game.team_id,
@@ -1136,12 +1135,7 @@ impl UiCallback {
                 target_player_id,
             } => Self::trade_players(*proposer_player_id, *target_player_id)(app),
             UiCallback::AcceptTrade { trade } => {
-                if let Err(e) = app
-                    .network_handler
-                    .as_mut()
-                    .ok_or(anyhow!("Network handler is not initialized"))?
-                    .accept_trade(&&app.world, trade.clone())
-                {
+                if let Err(e) = app.network_handler.accept_trade(&app.world, trade.clone()) {
                     let own_team = app.world.get_own_team_mut()?;
                     own_team.remove_trade(trade.proposer_player.id, trade.target_player.id);
                     return Err(e);
@@ -1152,10 +1146,7 @@ impl UiCallback {
                 Ok(None)
             }
             UiCallback::DeclineTrade { trade } => {
-                app.network_handler
-                    .as_mut()
-                    .ok_or(anyhow!("Network handler is not initialized"))?
-                    .decline_trade(trade.clone())?;
+                app.network_handler.decline_trade(trade.clone())?;
                 let own_team = app.world.get_own_team_mut()?;
                 own_team.remove_trade(trade.proposer_player.id, trade.target_player.id);
                 Ok(None)
@@ -1184,6 +1175,7 @@ impl UiCallback {
                 app.quit()?;
                 Ok(None)
             }
+            #[cfg(feature = "audio")]
             UiCallback::ToggleAudio => {
                 if let Some(player) = app.audio_player.as_mut() {
                     player.toggle_state()?;
@@ -1193,6 +1185,7 @@ impl UiCallback {
 
                 Ok(None)
             }
+            #[cfg(feature = "audio")]
             UiCallback::PreviousRadio => {
                 if let Some(player) = app.audio_player.as_mut() {
                     player.previous_radio_stream()?;
@@ -1201,6 +1194,7 @@ impl UiCallback {
                 }
                 Ok(None)
             }
+            #[cfg(feature = "audio")]
             UiCallback::NextRadio => {
                 if let Some(player) = app.audio_player.as_mut() {
                     player.next_radio_stream()?;
@@ -1251,11 +1245,7 @@ impl UiCallback {
                 let player = app.world.get_player_or_err(player_id)?;
                 let own_team = app.world.get_own_team()?;
                 let not_enough_players_for_game =
-                    if own_team.player_ids.len() - 1 < MIN_PLAYERS_PER_GAME {
-                        true
-                    } else {
-                        false
-                    };
+                    own_team.player_ids.len() - 1 < MIN_PLAYERS_PER_GAME;
                 app.ui.push_popup(PopupMessage::ReleasePlayer {
                     player_name: player.info.full_name(),
                     player_id: *player_id,
@@ -1337,7 +1327,7 @@ impl UiCallback {
                                     && p.peer_id.is_none()
                             })
                             .choose(rng)
-                            .map(|(&id, _)| id.clone()),
+                            .map(|(&id, _)| id),
 
                         TeamLocation::Exploring { around, .. } => app
                             .world
@@ -1347,7 +1337,7 @@ impl UiCallback {
                                 id != around && p.total_population() > 0 && p.peer_id.is_none()
                             })
                             .choose(rng)
-                            .map(|(&id, _)| id.clone()),
+                            .map(|(&id, _)| id),
                     };
                     if let Some(to) = portal_target_id {
                         let portal_target = app.world.get_planet_or_err(&to)?;
@@ -1411,7 +1401,7 @@ impl UiCallback {
             UiCallback::NextTrainingFocus { team_id } => Self::next_training_focus(*team_id)(app),
             UiCallback::TravelToPlanet { planet_id } => Self::travel_to_planet(*planet_id)(app),
             UiCallback::ExploreAroundPlanet { duration } => {
-                Self::explore_around_planet(duration.clone())(app)
+                Self::explore_around_planet(*duration)(app)
             }
             UiCallback::ZoomToPlanet {
                 planet_id,
@@ -1428,19 +1418,19 @@ impl UiCallback {
                 Self::name_and_accept_asteroid(name.clone(), filename.clone())(app)
             }
             UiCallback::SetSpaceshipUpgrade { upgrade } => {
-                Self::set_spaceship_upgrade(upgrade.clone())(app)
+                Self::set_spaceship_upgrade(*upgrade)(app)
             }
             UiCallback::UpgradeSpaceship { upgrade } => {
-                Self::upgrade_spaceship(upgrade.clone())(app)
+                Self::upgrade_spaceship(*upgrade)(app)
             }
             UiCallback::SetAsteroidUpgrade {
                 asteroid_id,
                 upgrade,
-            } => Self::set_asteroid_upgrade(*asteroid_id, upgrade.clone())(app),
+            } => Self::set_asteroid_upgrade(*asteroid_id, *upgrade)(app),
             UiCallback::UpgradeAsteroid {
                 asteroid_id,
                 upgrade,
-            } => Self::upgrade_asteroid(*asteroid_id, upgrade.clone())(app),
+            } => Self::upgrade_asteroid(*asteroid_id, *upgrade)(app),
             UiCallback::StartSpaceAdventure => {
                 app.ui.set_state(UiState::SpaceAdventure);
                 let mut own_team = app.world.get_own_team()?.clone();
@@ -1482,7 +1472,7 @@ impl UiCallback {
                 match own_team.current_location {
                     TeamLocation::OnPlanet { planet_id } => {
                         own_team.current_location = TeamLocation::OnSpaceAdventure {
-                            around: planet_id.clone(),
+                            around: planet_id,
                         }
                     }
                     _ => {
@@ -1556,7 +1546,7 @@ impl UiCallback {
                     }
                 }
 
-                if resources_gathered_text.len() == 0 {
+                if resources_gathered_text.is_empty() {
                     resources_gathered_text.push_str("No resources collected!")
                 } else {
                     resources_gathered_text.push_str("collected.")
@@ -1587,10 +1577,9 @@ impl UiCallback {
                     });
                 }
 
-                return Ok(Some(format!(
-                    "Team returned from space adventure:\n{}",
-                    resources_gathered_text
-                )));
+                Ok(Some(format!(
+                    "Team returned from space adventure:\n{resources_gathered_text}"
+                )))
             }
             UiCallback::SpaceMovePlayerLeft => {
                 if let Some(space) = app.world.space_adventure.as_mut() {
@@ -1693,7 +1682,7 @@ impl CallbackRegistry {
     ) {
         self.mouse_callbacks
             .entry(event_kind)
-            .or_insert_with(HashMap::new)
+            .or_default()
             .insert(rect, callback);
     }
 
