@@ -2,13 +2,9 @@ use super::{
     constants::{UiStyle, MAX_NAME_LENGTH, MIN_NAME_LENGTH},
     widgets::default_block,
 };
-use crate::{
-    types::Tick,
-    world::{AU, LIGHT_YEAR, SATOSHI_PER_BITCOIN},
-};
+use crate::world::{AU, LIGHT_YEAR, SATOSHI_PER_BITCOIN};
 use crossterm::event::{KeyCode, KeyEvent, KeyModifiers};
 use image::RgbaImage;
-use libp2p::PeerId;
 use ratatui::{
     style::{Color, Style},
     text::{Line, Span},
@@ -16,13 +12,6 @@ use ratatui::{
 };
 use tui_textarea::{Input, Key, TextArea};
 
-#[derive(Debug)]
-pub struct SwarmPanelEvent {
-    pub timestamp: Tick,
-    pub peer_id: Option<PeerId>,
-    pub text: String,
-}
-
 pub fn input_from_key_event(key: KeyEvent) -> Input {
     let ctrl = key.modifiers.contains(KeyModifiers::CONTROL);
     let alt = key.modifiers.contains(KeyModifiers::ALT);
@@ -151,7 +140,7 @@ pub fn format_satoshi(amount: u32) -> String {
     if amount >= 1_000_000 {
         let f_amount =
             (amount as f32 / SATOSHI_PER_BITCOIN as f32 * 1_000_000.0).round() / 1_000_000.0;
-        return format!("{:.4} BTC", f_amount);
+        return format!("{f_amount:.4} BTC");
     }
 
     format!("{amount} sat")
@@ -160,8 +149,8 @@ pub fn format_satoshi(amount: u32) -> String {
 pub fn format_au(amount: f32) -> String {
     const AU_PER_LIGHT_YEAR: f32 = LIGHT_YEAR as f32 / AU as f32;
     if amount >= 10_000.0 {
-        let f_amount = (amount as f32 / AU_PER_LIGHT_YEAR as f32 * 10_000.0).round() / 10_000.0;
-        return format!("{:.4} ly", f_amount);
+        let f_amount = (amount / AU_PER_LIGHT_YEAR * 10_000.0).round() / 10_000.0;
+        return format!("{f_amount:.4} ly");
     }
 
     format!("{amount:.4} AU")
