@@ -5,6 +5,7 @@ use serde::{Deserialize, Serialize};
 use strum::Display;
 use strum_macros::EnumIter;
 
+// FIXME: migrate to repr
 #[derive(
     Debug, Default, Clone, Copy, Display, Serialize, Deserialize, PartialEq, Hash, EnumIter,
 )]
@@ -31,10 +32,7 @@ impl JerseyStyle {
     }
 
     pub fn is_available_at_creation(&self) -> bool {
-        match self {
-            Self::Pirate => false,
-            _ => true,
-        }
+        !matches!(self, Self::Pirate)
     }
 }
 
