@@ -21,6 +21,11 @@ use std::{
 };
 use strum_macros::{Display, EnumIter};
 
+// Remve this imports once we remove the fallback logic for the asteroid upgrade serialization
+use serde::de::{self, Visitor};
+use serde::{Deserializer, Serializer};
+use std::fmt;
+
 const TRADE_DELTA_SCARCITY: f32 = 3.25;
 const TRADE_DELTA_BUY_SELL: f32 = 0.05;
 const RESOURCE_PRICE_REFRESH_RATE_MILLIS: Tick = 2 * HOURS;
@@ -44,9 +49,72 @@ pub enum PlanetType {
     Asteroid,
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize, Hash, EnumIter)]
+#[derive(Debug, Clone, Copy, PartialEq, Hash, EnumIter)]
+#[repr(u8)]
 pub enum AsteroidUpgradeTarget {
     TeleportationPad,
+    TortugaSpacePort,
+}
+
+// FIXME: remove in two releases
+impl Serialize for AsteroidUpgradeTarget {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        // Always serialize as u8 (new format)
+        serializer.serialize_u8(*self as u8)
+    }
+}
+
+impl<'de> Deserialize<'de> for AsteroidUpgradeTarget {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        struct EnumVisitor;
+
+        impl<'de> Visitor<'de> for EnumVisitor {
+            type Value = AsteroidUpgradeTarget;
+
+            fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                formatter.write_str("a u8 or string enum variant")
+            }
+
+            fn visit_u64<E>(self, v: u64) -> Result<Self::Value, E>
+            where
+                E: de::Error,
+            {
+                match v {
+                    0 => Ok(AsteroidUpgradeTarget::TeleportationPad),
+                    1 => Ok(AsteroidUpgradeTarget::TortugaSpacePort),
+                    _ => Err(E::custom(format!("unknown enum value {}", v))),
+                }
+            }
+
+            fn visit_str<E>(self, v: &str) -> Result<Self::Value, E>
+            where
+                E: de::Error,
+            {
+                match v {
+                    "TeleportationPad" => Ok(AsteroidUpgradeTarget::TeleportationPad),
+                    "TortugaSpacePort" => Ok(AsteroidUpgradeTarget::TortugaSpacePort),
+                    _ => Err(E::custom(format!("unknown enum variant {}", v))),
+                }
+            }
+        }
+
+        deserializer.deserialize_any(EnumVisitor)
+    }
+}
+
+impl AsteroidUpgradeTarget {
+    pub fn name(&self) -> &str {
+        match self {
+            Self::TeleportationPad => "Teleportation pad",
+            Self::TortugaSpacePort => "Tortuga space port",
+        }
+    }
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize, Hash)]
@@ -71,6 +139,9 @@ impl AsteroidUpgrade {
     pub fn description(&self) -> String {
         match self.target {
             AsteroidUpgradeTarget::TeleportationPad => "Building teleportation pad".to_string(),
+            AsteroidUpgradeTarget::TortugaSpacePort => {
+                "Building the Tortuga space port".to_string()
+            }
         }
     }
     pub fn cost(&self) -> Vec<(Resource, u32)> {
@@ -78,6 +149,9 @@ impl AsteroidUpgrade {
             AsteroidUpgradeTarget::TeleportationPad => {
                 vec![(Resource::SCRAPS, 125), (Resource::GOLD, 25)]
             }
+            AsteroidUpgradeTarget::TortugaSpacePort => {
+                vec![(Resource::SCRAPS, 250), (Resource::GOLD, 1000)]
+            }
         }
     }
 }
@@ -174,14 +248,14 @@ impl Planet {
     }
 
     pub fn total_population(&self) -> u32 {
-        self.populations.iter().map(|(_, p)| p).sum()
+        self.populations.values().sum()
     }
 
     pub fn random_population(&self, rng: &mut ChaCha8Rng) -> Option<Population> {
         let weights = self
             .populations
             .iter()
-            .map(|(pop, n)| (pop.clone(), n.clone()))
+            .map(|(pop, n)| (*pop, *n))
             .collect::<Vec<(Population, u32)>>();
 
         let dist = WeightedIndex::new(weights.iter().map(|(_, w)| w)).ok()?;
@@ -190,7 +264,7 @@ impl Planet {
 
     pub fn asteroid(name: String, filename: String, satellite_of: PlanetId) -> Self {
         let rng = &mut ChaCha8Rng::from_os_rng();
-        let revolution_period: usize = vec![120, 180, 360]
+        let revolution_period: usize = [120, 180, 360]
             .choose(rng)
             .copied()
             .expect("Should select a random value");
