@@ -16,6 +16,7 @@ use std::{fmt::Display, hash::Hash};
 use strum::IntoEnumIterator;
 use strum_macros::{Display, EnumIter};
 
+// FIXME: migrate to repr
 #[derive(Debug, Display, Clone, Copy, PartialEq, EnumIter)]
 pub enum SpaceshipStyle {
     Shuttle,
@@ -672,11 +673,8 @@ impl SpaceshipComponent for Storage {
         if self.next().cost() < self.cost() {
             return vec![];
         }
-        match self {
-            Storage::JesterNone => {
-                return vec![(Resource::SATOSHI, 30_000), (Resource::RUM, 50)];
-            }
-            _ => {}
+        if self == &Storage::JesterNone {
+            return vec![(Resource::SATOSHI, 30_000), (Resource::RUM, 50)];
         }
 
         let scraps_cost = (self.next().cost() - self.cost()) / 30;
