@@ -105,16 +105,24 @@ impl Team {
         }
     }
 
-    pub fn can_teleport_to(&self, to: &Planet) -> bool {
-        let rum_required = self.player_ids.len() as u32;
-        let has_rum = self.resources.value(&Resource::RUM) >= rum_required;
-
+    pub fn can_teleport_to(&self, to: &Planet) -> AppResult<()> {
         let has_teleportation_pad = self.home_planet_id == to.id
             || to
                 .upgrades
                 .contains(&AsteroidUpgradeTarget::TeleportationPad);
 
-        has_rum && has_teleportation_pad
+        if !has_teleportation_pad {
+            return Err(anyhow!("{} has no teleportation pad", to.name));
+        }
+
+        let rum_required = self.player_ids.len() as u32;
+        let has_rum = self.resources.value(&Resource::RUM) >= rum_required;
+
+        if !has_rum {
+            return Err(anyhow!("Not enough Rum! You need at least {rum_required}"));
+        }
+
+        Ok(())
     }
 
     pub fn add_sent_challenge(&mut self, challenge: Challenge) {
@@ -262,6 +270,11 @@ impl Team {
         (tiredness_iter.sum::<f32>() / n as f32).bound()
     }
 
+    pub fn is_on_player_planet(&self, player: &Player) -> bool {
+        // Player must be on same planet as team current_location
+        self.is_on_planet() == player.is_on_planet()
+    }
+
     pub fn can_add_player(&self, player: &Player) -> AppResult<()> {
         if player.team.is_some() {
             return Err(anyhow!("Already in a team"));
@@ -295,12 +308,11 @@ impl Team {
         Ok(())
     }
 
-    pub fn can_hire_player(&self, player: &Player) -> AppResult<()> {
-        self.can_add_player(player)?;
-
+    // This function is necessary for local teams to consider hiring a player (even if the crew is full).
+    pub fn can_consider_hiring_player(&self, player: &Player) -> AppResult<()> {
         let hiring_cost = player.hire_cost(self.reputation);
         if self.balance() < hiring_cost {
-            return Err(anyhow!("Not enough money {}", hiring_cost));
+            return Err(anyhow!("Not enough money {hiring_cost}"));
         }
 
         // Check player age is not above limit
@@ -311,6 +323,13 @@ impl Team {
         Ok(())
     }
 
+    pub fn can_hire_player(&self, player: &Player) -> AppResult<()> {
+        self.can_add_player(player)?;
+        self.can_consider_hiring_player(player)?;
+
+        Ok(())
+    }
+
     pub fn can_release_player(&self, player: &Player) -> AppResult<()> {
         if !self.player_ids.contains(&player.id) {
             return Err(anyhow!("Player is not in team"));
@@ -404,7 +423,7 @@ impl Team {
             return Err(anyhow!("{} is already playing", team.name));
         }
 
-        if self.sent_challenges.get(&team.id).is_some() {
+        if self.sent_challenges.contains_key(&team.id) {
             return Err(anyhow!("Already challenged {}", team.name));
         }
 
@@ -454,7 +473,7 @@ impl Team {
             return Err(anyhow!("Cannot travel to asteroid"));
         }
 
-        if self.player_ids.len() < 1 {
+        if self.player_ids.is_empty() {
             return Err(anyhow!("No pirate to travel"));
         }
 
@@ -479,15 +498,11 @@ impl Team {
             return Err(anyhow!("Planet {} is inhabitable", planet.name));
         }
 
-        let is_teleporting = if duration <= TELEPORT_MAX_DURATION {
-            true
-        } else {
-            false
-        };
+        let is_teleporting = duration <= TELEPORT_MAX_DURATION;
 
         if is_teleporting {
-            if !self.can_teleport_to(planet) {
-                return Err(anyhow!("Cannot teleport to planet {}", planet.name));
+            if let Err(e) = self.can_teleport_to(planet) {
+                return Err(anyhow!("Cannot teleport to planet {}: {e}", planet.name));
             }
         } else {
             // If we can't get there with full tank, than the planet is too far.
@@ -514,7 +529,7 @@ impl Team {
     }
 
     pub fn can_start_space_adventure(&self) -> AppResult<()> {
-        if self.player_ids.len() < 1 {
+        if self.player_ids.is_empty() {
             return Err(anyhow!("No pirate to explore"));
         }
 
@@ -608,7 +623,7 @@ impl Team {
         } else if amount < 0 {
             // Selling. Check if enough resource
             let current = self.resources.value(&resource);
-            if current < amount.abs() as u32 {
+            if current < amount.unsigned_abs() {
                 return Err(anyhow!("Not enough resource"));
             }
         }
@@ -725,7 +740,7 @@ impl Team {
         let mut bench = players
             .iter()
             .filter(|&p| !new_players.contains(&p.id))
-            .map(|&p| p)
+            .copied()
             .collect::<Vec<&Player>>();
         bench.sort_by(|a, b| {
             b.tiredness_weighted_rating()
